@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -117,10 +117,6 @@ impl Commit {
         &self.data
     }
 
-    pub fn is_open(&self) -> bool {
-        self.data.is_open
-    }
-
     pub fn is_empty(&self) -> bool {
         let parents = self.parents();
         // TODO: Perhaps the root commit should also be considered empty.
