@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -46,7 +46,6 @@ impl CommitBuilder {
             description: String::new(),
             author: signature.clone(),
             committer: signature,
-            is_open: false,
         };
         CommitBuilder {
             commit,
@@ -72,28 +71,6 @@ impl CommitBuilder {
         }
     }
 
-    pub fn for_open_commit(
-        settings: &UserSettings,
-        parent_id: CommitId,
-        tree_id: TreeId,
-    ) -> CommitBuilder {
-        let signature = settings.signature();
-        let commit = backend::Commit {
-            parents: vec![parent_id],
-            predecessors: vec![],
-            root_tree: tree_id,
-            change_id: new_change_id(),
-            description: String::new(),
-            author: signature.clone(),
-            committer: signature,
-            is_open: true,
-        };
-        CommitBuilder {
-            commit,
-            rewrite_source: None,
-        }
-    }
-
     pub fn set_parents(mut self, parents: Vec<CommitId>) -> Self {
         assert!(!parents.is_empty());
         self.commit.parents = parents;
@@ -125,11 +102,6 @@ impl CommitBuilder {
         self
     }
 
-    pub fn set_open(mut self, is_open: bool) -> Self {
-        self.commit.is_open = is_open;
-        self
-    }
-
     pub fn set_author(mut self, author: Signature) -> Self {
         self.commit.author = author;
         self
