@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,21 +12,21 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::collections::{BTreeMap, BTreeSet, HashSet};
-use std::fs::OpenOptions;
-use std::io::{Read, Write};
-use std::sync::Arc;
+use std::collections::{BTreeMap, HashSet};
+use std::default::Default;
+use std::path::PathBuf;
 
-use git2::{Oid, RemoteCallbacks};
+use git2::Oid;
 use itertools::Itertools;
 use thiserror::Error;
 
 use crate::backend::CommitId;
 use crate::commit::Commit;
-use crate::op_store::{OperationId, RefTarget};
-use crate::operation::Operation;
-use crate::repo::{MutableRepo, ReadonlyRepo, RepoRef};
-use crate::view::RefName;
+use crate::git_backend::NO_GC_REF_NAMESPACE;
+use crate::op_store;
+use crate::op_store::RefTarget;
+use crate::repo::MutableRepo;
+use crate::view::{RefName, View};
 
 #[derive(Error, Debug, PartialEq)]
 pub enum GitImportError {
@@ -51,6 +51,17 @@ fn parse_git_ref(ref_name: &str) -> Option<RefName> {
     }
 }
 
+fn prevent_gc(git_repo: &git2::Repository, id: &CommitId) {
+    git_repo
+        .reference(
+            &format!("{}{}", NO_GC_REF_NAMESPACE, id.hex()),
+            Oid::from_bytes(id.as_bytes()).unwrap(),
+            true,
+            "used by jj",
+        )
+        .unwrap();
+}
+
 /// Reflect changes made in the underlying Git repo in the Jujutsu repo.
 pub fn import_refs(
     mut_repo: &mut MutableRepo,
@@ -58,11 +69,31 @@ pub fn import_refs(
 ) -> Result<(), GitImportError> {
     let store = mut_repo.store().clone();
     let mut existing_git_refs = mut_repo.view().git_refs().clone();
-    let old_git_heads = existing_git_refs
+    let mut old_git_heads = existing_git_refs
         .values()
         .flat_map(|old_target| old_target.adds())
         .collect_vec();
+    if let Some(old_git_head) = mut_repo.view().git_head() {
+        old_git_heads.push(old_git_head);
+    }
+
     let mut new_git_heads = HashSet::new();
+    // TODO: Should this be a separate function? We may not always want to import
+    // the Git HEAD (and add it to our set of heads).
+    if let Ok(head_git_commit) = git_repo
+        .head()
+        .and_then(|head_ref| head_ref.peel_to_commit())
+    {
+        let head_commit_id = CommitId::from_bytes(head_git_commit.id().as_bytes());
+        let head_commit = store.get_commit(&head_commit_id).unwrap();
+        new_git_heads.insert(head_commit_id.clone());
+        prevent_gc(git_repo, &head_commit_id);
+        mut_repo.add_head(&head_commit);
+        mut_repo.set_git_head(head_commit_id);
+    } else {
+        mut_repo.clear_git_head();
+    }
+
     let mut changed_git_refs = BTreeMap::new();
     let git_refs = git_repo.references()?;
     for git_ref in git_refs {
@@ -91,10 +122,11 @@ pub fn import_refs(
         new_git_heads.insert(id.clone());
         // TODO: Make it configurable which remotes are publishing and update public
         // heads here.
-        mut_repo.set_git_ref(full_name.clone(), RefTarget::Normal(id.clone()));
         let old_target = existing_git_refs.remove(&full_name);
         let new_target = Some(RefTarget::Normal(id.clone()));
         if new_target != old_target {
+            prevent_gc(git_repo, &id);
+            mut_repo.set_git_ref(full_name.clone(), RefTarget::Normal(id.clone()));
             let commit = store.get_commit(&id).unwrap();
             mut_repo.add_head(&commit);
             changed_git_refs.insert(full_name, (old_target, new_target));
@@ -132,23 +164,13 @@ pub fn import_refs(
         .walk_revs(&old_git_heads, &new_git_heads)
         .map(|entry| entry.commit_id())
         .collect_vec();
+    let root_commit_id = mut_repo.store().root_commit_id().clone();
     for abandoned_commit in abandoned_commits {
-        mut_repo.record_abandoned_commit(abandoned_commit);
+        if abandoned_commit != root_commit_id {
+            mut_repo.record_abandoned_commit(abandoned_commit);
+        }
     }
 
-    // TODO: Should this be a separate function? We may not always want to import
-    // the Git HEAD (and add it to our set of heads).
-    if let Ok(head_git_commit) = git_repo
-        .head()
-        .and_then(|head_ref| head_ref.peel_to_commit())
-    {
-        let head_commit_id = CommitId::from_bytes(head_git_commit.id().as_bytes());
-        let head_commit = store.get_commit(&head_commit_id).unwrap();
-        mut_repo.add_head(&head_commit);
-        mut_repo.set_git_head(head_commit_id);
-    } else {
-        mut_repo.clear_git_head();
-    }
     Ok(())
 }
 
@@ -156,106 +178,150 @@ pub fn import_refs(
 pub enum GitExportError {
     #[error("Cannot export conflicted branch '{0}'")]
     ConflictedBranch(String),
-    #[error("Unexpected git error when exporting refs: {0}")]
+    #[error("Failed to read export state: {0}")]
+    ReadStateError(String),
+    #[error("Failed to write export state: {0}")]
+    WriteStateError(String),
+    #[error("Git error: {0}")]
     InternalGitError(#[from] git2::Error),
 }
 
-/// Reflect changes between two Jujutsu repo states in the underlying Git repo.
-pub fn export_changes(
-    old_repo: RepoRef,
-    new_repo: RepoRef,
+/// Reflect changes made in the Jujutsu repo compared to our current view of the
+/// Git repo in `mut_repo.view().git_refs()`. Returns a list of names of
+/// branches that failed to export.
+// TODO: Also indicate why we failed to export these branches
+pub fn export_refs(
+    mut_repo: &mut MutableRepo,
     git_repo: &git2::Repository,
-) -> Result<(), GitExportError> {
-    let old_view = old_repo.view();
-    let new_view = new_repo.view();
-    let old_branches: HashSet<_> = old_view.branches().keys().cloned().collect();
-    let new_branches: HashSet<_> = new_view.branches().keys().cloned().collect();
-    // TODO: Check that the ref is not pointed to by any worktree's HEAD.
-    let mut active_branches = HashSet::new();
-    if let Ok(head_ref) = git_repo.find_reference("HEAD") {
-        if let Some(head_target) = head_ref.symbolic_target() {
-            active_branches.insert(head_target.to_string());
+) -> Result<Vec<String>, GitExportError> {
+    let mut old_view = View::new(op_store::View::default());
+    for (git_ref, git_ref_target) in mut_repo.view().git_refs() {
+        if let Some(branch_name) = git_ref.strip_prefix("refs/heads/") {
+            old_view.set_local_branch(branch_name.to_string(), git_ref_target.clone());
         }
     }
-    let mut detach_head = false;
-    // First find the changes we want need to make and then make them all at once to
-    // reduce the risk of making some changes before we fail.
-    let mut refs_to_update = BTreeMap::new();
-    let mut refs_to_delete = BTreeSet::new();
+    let new_view = mut_repo.view();
+    let old_branches: HashSet<_> = old_view.branches().keys().cloned().collect();
+    let new_branches: HashSet<_> = new_view.branches().keys().cloned().collect();
+    let mut branches_to_update = BTreeMap::new();
+    let mut branches_to_delete = BTreeMap::new();
+    // First find the changes we want need to make without modifying mut_repo
+    let mut failed_branches = vec![];
     for branch_name in old_branches.union(&new_branches) {
         let old_branch = old_view.get_local_branch(branch_name);
         let new_branch = new_view.get_local_branch(branch_name);
         if new_branch == old_branch {
             continue;
         }
-        let git_ref_name = format!("refs/heads/{}", branch_name);
+        let old_oid = match old_branch {
+            None => None,
+            Some(RefTarget::Normal(id)) => Some(Oid::from_bytes(id.as_bytes()).unwrap()),
+            Some(RefTarget::Conflict { .. }) => {
+                // The old git ref should only be a conflict if there were concurrent import
+                // operations while the value changed. Don't overwrite these values.
+                failed_branches.push(branch_name.clone());
+                continue;
+            }
+        };
         if let Some(new_branch) = new_branch {
             match new_branch {
                 RefTarget::Normal(id) => {
-                    refs_to_update.insert(
-                        git_ref_name.clone(),
-                        Oid::from_bytes(id.as_bytes()).unwrap(),
-                    );
+                    let new_oid = Oid::from_bytes(id.as_bytes());
+                    branches_to_update.insert(branch_name.clone(), (old_oid, new_oid.unwrap()));
                 }
                 RefTarget::Conflict { .. } => {
-                    return Err(GitExportError::ConflictedBranch(branch_name.to_string()));
+                    // Skip conflicts and leave the old value in git_refs
+                    continue;
                 }
             }
         } else {
-            refs_to_delete.insert(git_ref_name.clone());
-        }
-        if active_branches.contains(&git_ref_name) {
-            detach_head = true;
+            branches_to_delete.insert(branch_name.clone(), old_oid.unwrap());
         }
     }
-    if detach_head {
-        if let Ok(head_ref) = git_repo.find_reference("HEAD") {
-            if let Ok(current_git_commit) = head_ref.peel_to_commit() {
-                git_repo.set_head_detached(current_git_commit.id())?;
+    // TODO: Also check other worktrees' HEAD.
+    if let Ok(head_ref) = git_repo.find_reference("HEAD") {
+        if let (Some(head_git_ref), Ok(current_git_commit)) =
+            (head_ref.symbolic_target(), head_ref.peel_to_commit())
+        {
+            if let Some(branch_name) = head_git_ref.strip_prefix("refs/heads/") {
+                let detach_head =
+                    if let Some((_old_oid, new_oid)) = branches_to_update.get(branch_name) {
+                        *new_oid != current_git_commit.id()
+                    } else {
+                        branches_to_delete.contains_key(branch_name)
+                    };
+                if detach_head {
+                    git_repo.set_head_detached(current_git_commit.id())?;
+                }
             }
         }
     }
-    for (git_ref_name, new_target) in refs_to_update {
-        git_repo.reference(&git_ref_name, new_target, true, "export from jj")?;
-    }
-    for git_ref_name in refs_to_delete {
-        if let Ok(mut git_ref) = git_repo.find_reference(&git_ref_name) {
-            git_ref.delete()?;
+    for (branch_name, old_oid) in branches_to_delete {
+        let git_ref_name = format!("refs/heads/{}", branch_name);
+        let success = if let Ok(mut git_ref) = git_repo.find_reference(&git_ref_name) {
+            if git_ref.target() == Some(old_oid) {
+                // The branch has not been updated by git, so go ahead and delete it
+                git_ref.delete().is_ok()
+            } else {
+                // The branch was updated by git
+                false
+            }
+        } else {
+            // The branch is already deleted
+            true
+        };
+        if success {
+            mut_repo.remove_git_ref(&git_ref_name);
+        } else {
+            failed_branches.push(branch_name);
         }
     }
-    Ok(())
-}
-
-/// Reflect changes made in the Jujutsu repo since last export in the underlying
-/// Git repo. If this is the first export, nothing will be exported. The
-/// exported state's operation ID is recorded in the repo
-/// (`.jj/repo/git_export_operation_id`).
-pub fn export_refs(
-    repo: &Arc<ReadonlyRepo>,
-    git_repo: &git2::Repository,
-) -> Result<(), GitExportError> {
-    let last_export_path = repo.repo_path().join("git_export_operation_id");
-    if let Ok(mut last_export_file) = OpenOptions::new().read(true).open(&last_export_path) {
-        let mut buf = vec![];
-        last_export_file.read_to_end(&mut buf).unwrap();
-        let last_export_op_id = OperationId::from_hex(String::from_utf8(buf).unwrap().as_str());
-        let loader = repo.loader();
-        let op_store = loader.op_store();
-        let last_export_store_op = op_store.read_operation(&last_export_op_id).unwrap();
-        let last_export_op =
-            Operation::new(op_store.clone(), last_export_op_id, last_export_store_op);
-        let old_repo = repo.loader().load_at(&last_export_op);
-        export_changes(old_repo.as_repo_ref(), repo.as_repo_ref(), git_repo)?;
+    for (branch_name, (old_oid, new_oid)) in branches_to_update {
+        let git_ref_name = format!("refs/heads/{}", branch_name);
+        let success = match old_oid {
+            None => {
+                if let Ok(git_ref) = git_repo.find_reference(&git_ref_name) {
+                    // The branch was added in jj and in git. Iff git already pointed it to our
+                    // desired target, we're good
+                    git_ref.target() == Some(new_oid)
+                } else {
+                    // The branch was added in jj but still doesn't exist in git, so add it
+                    git_repo
+                        .reference(&git_ref_name, new_oid, true, "export from jj")
+                        .is_ok()
+                }
+            }
+            Some(old_oid) => {
+                // The branch was modified in jj. We can use libgit2's API for updating under a
+                // lock.
+                if git_repo
+                    .reference_matching(&git_ref_name, new_oid, true, old_oid, "export from jj")
+                    .is_ok()
+                {
+                    // Successfully updated from old_oid to new_oid (unchanged in git)
+                    true
+                } else {
+                    // The reference was probably updated in git
+                    if let Ok(git_ref) = git_repo.find_reference(&git_ref_name) {
+                        // Iff it was updated to our desired target, we still consider it a success
+                        git_ref.target() == Some(new_oid)
+                    } else {
+                        // The reference was deleted in git
+                        false
+                    }
+                }
+            }
+        };
+        if success {
+            mut_repo.set_git_ref(
+                git_ref_name,
+                RefTarget::Normal(CommitId::from_bytes(new_oid.as_bytes())),
+            );
+        } else {
+            failed_branches.push(branch_name);
+        }
     }
-    if let Ok(mut last_export_file) = OpenOptions::new()
-        .write(true)
-        .create(true)
-        .open(&last_export_path)
-    {
-        let buf = repo.op_id().hex().as_bytes().to_vec();
-        last_export_file.write_all(&buf).unwrap();
-    }
-    Ok(())
+    Ok(failed_branches)
 }
 
 #[derive(Error, Debug, PartialEq)]
@@ -267,10 +333,12 @@ pub enum GitFetchError {
     InternalGitError(#[from] git2::Error),
 }
 
+#[tracing::instrument(skip(mut_repo, git_repo, callbacks))]
 pub fn fetch(
     mut_repo: &mut MutableRepo,
     git_repo: &git2::Repository,
     remote_name: &str,
+    callbacks: RemoteCallbacks<'_>,
 ) -> Result<Option<String>, GitFetchError> {
     let mut remote =
         git_repo
@@ -288,11 +356,14 @@ pub fn fetch(
     let mut proxy_options = git2::ProxyOptions::new();
     proxy_options.auto();
     fetch_options.proxy_options(proxy_options);
-    let callbacks = create_remote_callbacks();
+    let callbacks = callbacks.into_git();
     fetch_options.remote_callbacks(callbacks);
     let refspec: &[&str] = &[];
+    tracing::debug!("remote.download");
     remote.download(refspec, Some(&mut fetch_options))?;
+    tracing::debug!("remote.update_tips");
     remote.update_tips(None, false, git2::AutotagOption::Unspecified, None)?;
+    tracing::debug!("remote.prune");
     remote.prune(None)?;
     // TODO: We could make it optional to get the default branch since we only care
     // about it on clone.
@@ -302,11 +373,14 @@ pub fn fetch(
             // LocalBranch here is the local branch on the remote, so it's really the remote
             // branch
             if let Some(RefName::LocalBranch(branch_name)) = parse_git_ref(default_ref) {
+                tracing::debug!(default_branch = branch_name);
                 default_branch = Some(branch_name);
             }
         }
     }
+    tracing::debug!("remote.disconnect");
     remote.disconnect()?;
+    tracing::debug!("import_refs");
     import_refs(mut_repo, git_repo).map_err(|err| match err {
         GitImportError::InternalGitError(source) => GitFetchError::InternalGitError(source),
     })?;
@@ -336,6 +410,7 @@ pub fn push_commit(
     // It's a blunt "force" option instead until git2-rs supports the "push negotiation" callback
     // (https://github.com/rust-lang/git2-rs/issues/733).
     force: bool,
+    callbacks: RemoteCallbacks<'_>,
 ) -> Result<(), GitPushError> {
     push_updates(
         git_repo,
@@ -345,6 +420,7 @@ pub fn push_commit(
             force,
             new_target: Some(target.id().clone()),
         }],
+        callbacks,
     )
 }
 
@@ -361,6 +437,7 @@ pub fn push_updates(
     git_repo: &git2::Repository,
     remote_name: &str,
     updates: &[GitRefUpdate],
+    callbacks: RemoteCallbacks<'_>,
 ) -> Result<(), GitPushError> {
     let mut temp_refs = vec![];
     let mut qualified_remote_refs = vec![];
@@ -386,7 +463,13 @@ pub fn push_updates(
             refspecs.push(format!(":{}", update.qualified_name));
         }
     }
-    let result = push_refs(git_repo, remote_name, &qualified_remote_refs, &refspecs);
+    let result = push_refs(
+        git_repo,
+        remote_name,
+        &qualified_remote_refs,
+        &refspecs,
+        callbacks,
+    );
     for mut temp_ref in temp_refs {
         // TODO: Figure out how to do the equivalent of absl::Cleanup for
         // temp_ref.delete().
@@ -407,6 +490,7 @@ fn push_refs(
     remote_name: &str,
     qualified_remote_refs: &[&str],
     refspecs: &[String],
+    callbacks: RemoteCallbacks<'_>,
 ) -> Result<(), GitPushError> {
     let mut remote =
         git_repo
@@ -425,7 +509,7 @@ fn push_refs(
     let mut proxy_options = git2::ProxyOptions::new();
     proxy_options.auto();
     push_options.proxy_options(proxy_options);
-    let mut callbacks = create_remote_callbacks();
+    let mut callbacks = callbacks.into_git();
     callbacks.push_update_reference(|refname, status| {
         // The status is Some if the ref update was rejected
         if status.is_none() {
@@ -456,23 +540,102 @@ fn push_refs(
     }
 }
 
-fn create_remote_callbacks() -> RemoteCallbacks<'static> {
-    let mut callbacks = git2::RemoteCallbacks::new();
-    // TODO: We should expose the callbacks to the caller instead -- the library
-    // crate shouldn't look in $HOME etc.
-    callbacks.credentials(|_url, username_from_url, allowed_types| {
-        if allowed_types.contains(git2::CredentialType::SSH_KEY) {
-            if std::env::var("SSH_AUTH_SOCK").is_ok() || std::env::var("SSH_AGENT_PID").is_ok() {
-                return git2::Cred::ssh_key_from_agent(username_from_url.unwrap());
-            }
-            if let Ok(home_dir) = std::env::var("HOME") {
-                let key_path = std::path::Path::new(&home_dir).join(".ssh").join("id_rsa");
-                if key_path.is_file() {
-                    return git2::Cred::ssh_key(username_from_url.unwrap(), None, &key_path, None);
+#[non_exhaustive]
+#[derive(Default)]
+#[allow(clippy::type_complexity)]
+pub struct RemoteCallbacks<'a> {
+    pub progress: Option<&'a mut dyn FnMut(&Progress)>,
+    pub get_ssh_key: Option<&'a mut dyn FnMut(&str) -> Option<PathBuf>>,
+    pub get_password: Option<&'a mut dyn FnMut(&str, &str) -> Option<String>>,
+    pub get_username_password: Option<&'a mut dyn FnMut(&str) -> Option<(String, String)>>,
+}
+
+impl<'a> RemoteCallbacks<'a> {
+    fn into_git(mut self) -> git2::RemoteCallbacks<'a> {
+        let mut callbacks = git2::RemoteCallbacks::new();
+        if let Some(progress_cb) = self.progress {
+            callbacks.transfer_progress(move |progress| {
+                progress_cb(&Progress {
+                    bytes_downloaded: if progress.received_objects() < progress.total_objects() {
+                        Some(progress.received_bytes() as u64)
+                    } else {
+                        None
+                    },
+                    overall: (progress.indexed_objects() + progress.indexed_deltas()) as f32
+                        / (progress.total_objects() + progress.total_deltas()) as f32,
+                });
+                true
+            });
+        }
+        // TODO: We should expose the callbacks to the caller instead -- the library
+        // crate shouldn't read environment variables.
+        callbacks.credentials(move |url, username_from_url, allowed_types| {
+            let span = tracing::debug_span!("RemoteCallbacks.credentials");
+            let _ = span.enter();
+
+            let git_config = git2::Config::open_default();
+            let credential_helper = git_config
+                .and_then(|conf| git2::Cred::credential_helper(&conf, url, username_from_url));
+            if let Ok(creds) = credential_helper {
+                tracing::debug!("using credential_helper");
+                return Ok(creds);
+            } else if let Some(username) = username_from_url {
+                if allowed_types.contains(git2::CredentialType::SSH_KEY) {
+                    if std::env::var("SSH_AUTH_SOCK").is_ok()
+                        || std::env::var("SSH_AGENT_PID").is_ok()
+                    {
+                        tracing::debug!(username, "using ssh_key_from_agent");
+                        return git2::Cred::ssh_key_from_agent(username).map_err(|err| {
+                            tracing::error!(err = %err);
+                            err
+                        });
+                    }
+                    if let Some(ref mut cb) = self.get_ssh_key {
+                        if let Some(path) = cb(username) {
+                            tracing::debug!(username, path = ?path, "using ssh_key");
+                            return git2::Cred::ssh_key(username, None, &path, None).map_err(
+                                |err| {
+                                    tracing::error!(err = %err);
+                                    err
+                                },
+                            );
+                        }
+                    }
+                }
+                if allowed_types.contains(git2::CredentialType::USER_PASS_PLAINTEXT) {
+                    if let Some(ref mut cb) = self.get_password {
+                        if let Some(pw) = cb(url, username) {
+                            tracing::debug!(
+                                username,
+                                "using userpass_plaintext with username from url"
+                            );
+                            return git2::Cred::userpass_plaintext(username, &pw).map_err(|err| {
+                                tracing::error!(err = %err);
+                                err
+                            });
+                        }
+                    }
+                }
+            } else if allowed_types.contains(git2::CredentialType::USER_PASS_PLAINTEXT) {
+                if let Some(ref mut cb) = self.get_username_password {
+                    if let Some((username, pw)) = cb(url) {
+                        tracing::debug!(username, "using userpass_plaintext");
+                        return git2::Cred::userpass_plaintext(&username, &pw).map_err(|err| {
+                            tracing::error!(err = %err);
+                            err
+                        });
+                    }
                 }
             }
-        }
-        git2::Cred::default()
-    });
-    callbacks
+            tracing::debug!("using default");
+            git2::Cred::default()
+        });
+        callbacks
+    }
+}
+
+pub struct Progress {
+    /// `Some` iff data transfer is currently in progress
+    pub bytes_downloaded: Option<u64>,
+    pub overall: f32,
 }
