@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -33,7 +33,7 @@ use crate::stacked_table::{TableSegment, TableStore};
 
 const HASH_LENGTH: usize = 20;
 /// Ref namespace used only for preventing GC.
-const NO_GC_REF_NAMESPACE: &str = "refs/jj/keep/";
+pub const NO_GC_REF_NAMESPACE: &str = "refs/jj/keep/";
 const CONFLICT_SUFFIX: &str = ".jjconflict";
 
 impl From<git2::Error> for BackendError {
@@ -65,7 +65,7 @@ impl GitBackend {
     }
 
     pub fn init_internal(store_path: &Path) -> Self {
-        let git_repo = git2::Repository::init_bare(&store_path.join("git")).unwrap();
+        let git_repo = git2::Repository::init_bare(store_path.join("git")).unwrap();
         let extra_path = store_path.join("extra");
         std::fs::create_dir(&extra_path).unwrap();
         let mut git_target_file = File::create(store_path.join("git_target")).unwrap();
@@ -125,7 +125,6 @@ fn signature_to_git(signature: &Signature) -> git2::Signature {
 
 fn serialize_extras(commit: &Commit) -> Vec<u8> {
     let mut proto = crate::protos::store::Commit::new();
-    proto.is_open = commit.is_open;
     proto.change_id = commit.change_id.to_bytes();
     for predecessor in &commit.predecessors {
         proto.predecessors.push(predecessor.to_bytes());
@@ -136,7 +135,6 @@ fn serialize_extras(commit: &Commit) -> Vec<u8> {
 fn deserialize_extras(commit: &mut Commit, bytes: &[u8]) {
     let mut cursor = Cursor::new(bytes);
     let proto: crate::protos::store::Commit = Message::parse_from_reader(&mut cursor).unwrap();
-    commit.is_open = proto.is_open;
     commit.change_id = ChangeId::new(proto.change_id);
     for predecessor in &proto.predecessors {
         commit.predecessors.push(CommitId::from_bytes(predecessor));
@@ -174,7 +172,7 @@ impl Backend for GitBackend {
 
     fn git_repo(&self) -> Option<git2::Repository> {
         let path = self.repo.lock().unwrap().path().to_owned();
-        Some(git2::Repository::open(&path).unwrap())
+        Some(git2::Repository::open(path).unwrap())
     }
 
     fn read_file(&self, _path: &RepoPath, id: &FileId) -> BackendResult<Box<dyn Read>> {
@@ -254,7 +252,7 @@ impl Backend for GitBackend {
                         } else {
                             (
                                 name,
-                                TreeValue::Normal {
+                                TreeValue::File {
                                     id,
                                     executable: false,
                                 },
@@ -265,7 +263,7 @@ impl Backend for GitBackend {
                         let id = FileId::from_bytes(entry.id().as_bytes());
                         (
                             name,
-                            TreeValue::Normal {
+                            TreeValue::File {
                                 id,
                                 executable: true,
                             },
@@ -294,11 +292,11 @@ impl Backend for GitBackend {
         for entry in contents.entries() {
             let name = entry.name().string();
             let (name, id, filemode) = match entry.value() {
-                TreeValue::Normal {
+                TreeValue::File {
                     id,
                     executable: false,
                 } => (name, id.as_bytes(), 0o100644),
-                TreeValue::Normal {
+                TreeValue::File {
                     id,
                     executable: true,
                 } => (name, id.as_bytes(), 0o100755),
@@ -390,7 +388,6 @@ impl Backend for GitBackend {
             description,
             author,
             committer,
-            is_open: false,
         };
 
         let table = self.extra_metadata_store.get_head().map_err(|err| {
@@ -486,7 +483,7 @@ fn conflict_part_from_json(json: &serde_json::Value) -> ConflictPart {
 
 fn tree_value_to_json(value: &TreeValue) -> serde_json::Value {
     match value {
-        TreeValue::Normal { id, executable } => serde_json::json!({
+        TreeValue::File { id, executable } => serde_json::json!({
              "file": {
                  "id": id.hex(),
                  "executable": executable,
@@ -509,7 +506,7 @@ fn tree_value_to_json(value: &TreeValue) -> serde_json::Value {
 
 fn tree_value_from_json(json: &serde_json::Value) -> TreeValue {
     if let Some(json_file) = json.get("file") {
-        TreeValue::Normal {
+        TreeValue::File {
             id: FileId::new(bytes_vec_from_json(json_file.get("id").unwrap())),
             executable: json_file.get("executable").unwrap().as_bool().unwrap(),
         }
@@ -535,7 +532,6 @@ mod tests {
 
     use super::*;
     use crate::backend::{FileId, MillisSinceEpoch};
-    use crate::testutils;
 
     #[test]
     fn read_plain_git_commit() {
@@ -591,7 +587,6 @@ mod tests {
         assert_eq!(commit.parents, vec![CommitId::from_bytes(&[0; 20])]);
         assert_eq!(commit.predecessors, vec![]);
         assert_eq!(commit.root_tree.as_bytes(), root_tree_id.as_bytes());
-        assert!(!commit.is_open);
         assert_eq!(commit.description, "git commit message");
         assert_eq!(commit.author.name, "git author");
         assert_eq!(commit.author.email, "git.author@example.com");
@@ -629,14 +624,14 @@ mod tests {
                 &TreeId::from_bytes(dir_tree_id.as_bytes()),
             )
             .unwrap();
-        let mut files = dir_tree.entries();
-        let normal_file = files.next().unwrap();
-        let symlink = files.next().unwrap();
-        assert_eq!(files.next(), None);
-        assert_eq!(normal_file.name().as_str(), "normal");
+        let mut entries = dir_tree.entries();
+        let file = entries.next().unwrap();
+        let symlink = entries.next().unwrap();
+        assert_eq!(entries.next(), None);
+        assert_eq!(file.name().as_str(), "normal");
         assert_eq!(
-            normal_file.value(),
-            &TreeValue::Normal {
+            file.value(),
+            &TreeValue::File {
                 id: FileId::from_bytes(blob1.as_bytes()),
                 executable: false
             }
@@ -668,7 +663,6 @@ mod tests {
             description: "initial".to_string(),
             author: signature.clone(),
             committer: signature,
-            is_open: false,
         };
         let commit_id = store.write_commit(&commit).unwrap();
         let git_refs = store
@@ -704,7 +698,6 @@ mod tests {
             description: "initial".to_string(),
             author: signature.clone(),
             committer: signature,
-            is_open: false,
         };
         let commit_id1 = store.write_commit(&commit1).unwrap();
         let mut commit2 = commit1;
