@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -33,6 +33,9 @@ use crate::backend::{ChangeId, CommitId};
 use crate::commit::Commit;
 use crate::file_util::persist_content_addressed_temp_file;
 #[cfg(not(feature = "map_first_last"))]
+// This import is used on Rust 1.60, but not on recent version.
+// TODO: Remove it when our MSRV becomes recent enough.
+#[allow(unused_imports)]
 use crate::nightly_shims::BTreeSetExt;
 
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone, Copy, Hash)]
@@ -280,7 +283,7 @@ impl HexPrefix {
             let bytes = hex::decode(&self.0).unwrap();
             (CommitId::new(bytes.clone()), CommitId::new(bytes))
         } else {
-            let min_bytes = hex::decode(&(self.0.clone() + "0")).unwrap();
+            let min_bytes = hex::decode(self.0.clone() + "0").unwrap();
             let prefix = min_bytes[0..min_bytes.len() - 1].to_vec();
             (CommitId::new(prefix), CommitId::new(min_bytes))
         }
@@ -559,7 +562,7 @@ impl MutableIndex {
         let mut temp_file = NamedTempFile::new_in(&dir)?;
         let file = temp_file.as_file_mut();
         file.write_all(&buf)?;
-        persist_content_addressed_temp_file(temp_file, &index_file_path)?;
+        persist_content_addressed_temp_file(temp_file, index_file_path)?;
 
         let mut cursor = Cursor::new(&buf);
         ReadonlyIndex::load_from(&mut cursor, dir, index_file_id_hex, hash_length).map_err(|err| {
@@ -967,15 +970,27 @@ impl PartialOrd for IndexEntryByGeneration<'_> {
 #[derive(Clone, Eq, PartialEq, Ord, PartialOrd)]
 struct RevWalkWorkItem<'a> {
     entry: IndexEntryByPosition<'a>,
-    wanted: bool,
+    state: RevWalkWorkItemState,
+}
+
+#[derive(Clone, Copy, Debug, Eq, Ord, PartialEq, PartialOrd)]
+enum RevWalkWorkItemState {
+    // Order matters: Unwanted should appear earlier in the max-heap.
+    Wanted,
+    Unwanted,
+}
+
+impl RevWalkWorkItem<'_> {
+    fn is_wanted(&self) -> bool {
+        self.state == RevWalkWorkItemState::Wanted
+    }
 }
 
 #[derive(Clone)]
 pub struct RevWalk<'a> {
     index: CompositeIndex<'a>,
     items: BinaryHeap<RevWalkWorkItem<'a>>,
-    wanted_boundary_set: HashSet<IndexPosition>,
-    unwanted_boundary_set: HashSet<IndexPosition>,
+    unwanted_count: usize,
 }
 
 impl<'a> RevWalk<'a> {
@@ -983,29 +998,46 @@ impl<'a> RevWalk<'a> {
         Self {
             index,
             items: BinaryHeap::new(),
-            wanted_boundary_set: HashSet::new(),
-            unwanted_boundary_set: HashSet::new(),
+            unwanted_count: 0,
         }
     }
 
     fn add_wanted(&mut self, pos: IndexPosition) {
-        if !self.wanted_boundary_set.insert(pos) {
-            return;
-        }
         self.items.push(RevWalkWorkItem {
             entry: IndexEntryByPosition(self.index.entry_by_pos(pos)),
-            wanted: true,
+            state: RevWalkWorkItemState::Wanted,
         });
     }
 
     fn add_unwanted(&mut self, pos: IndexPosition) {
-        if !self.unwanted_boundary_set.insert(pos) {
-            return;
-        }
         self.items.push(RevWalkWorkItem {
             entry: IndexEntryByPosition(self.index.entry_by_pos(pos)),
-            wanted: false,
+            state: RevWalkWorkItemState::Unwanted,
         });
+        self.unwanted_count += 1;
+    }
+
+    fn pop(&mut self) -> Option<RevWalkWorkItem<'a>> {
+        if let Some(x) = self.items.pop() {
+            self.unwanted_count -= !x.is_wanted() as usize;
+            Some(x)
+        } else {
+            None
+        }
+    }
+
+    fn pop_eq(&mut self, entry: &IndexEntry<'_>) -> Option<RevWalkWorkItem<'a>> {
+        if let Some(x) = self.items.peek() {
+            (&x.entry.0 == entry).then(|| self.pop().unwrap())
+        } else {
+            None
+        }
+    }
+
+    fn skip_while_eq(&mut self, entry: &IndexEntry<'_>) {
+        while self.pop_eq(entry).is_some() {
+            continue;
+        }
     }
 }
 
@@ -1013,23 +1045,28 @@ impl<'a> Iterator for RevWalk<'a> {
     type Item = IndexEntry<'a>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        while let Some(item) = self.items.pop() {
-            if item.wanted {
-                self.wanted_boundary_set.remove(&item.entry.0.pos);
-                if self.unwanted_boundary_set.contains(&item.entry.0.pos) {
-                    continue;
-                }
+        while let Some(item) = self.pop() {
+            self.skip_while_eq(&item.entry.0);
+            if item.is_wanted() {
                 for parent_pos in item.entry.0.parent_positions() {
                     self.add_wanted(parent_pos);
                 }
                 return Some(item.entry.0);
+            } else if self.items.len() == self.unwanted_count {
+                // No more wanted entries to walk
+                debug_assert!(!self.items.iter().any(|x| x.is_wanted()));
+                return None;
             } else {
-                self.unwanted_boundary_set.remove(&item.entry.0.pos);
                 for parent_pos in item.entry.0.parent_positions() {
                     self.add_unwanted(parent_pos);
                 }
             }
         }
+
+        debug_assert_eq!(
+            self.items.iter().filter(|x| !x.is_wanted()).count(),
+            self.unwanted_count
+        );
         None
     }
 }
@@ -1307,7 +1344,7 @@ impl ReadonlyIndex {
             file.read_exact(&mut parent_filename_bytes)?;
             let parent_filename = String::from_utf8(parent_filename_bytes).unwrap();
             let parent_file_path = dir.join(&parent_filename);
-            let mut index_file = File::open(&parent_file_path).unwrap();
+            let mut index_file = File::open(parent_file_path).unwrap();
             let parent_file =
                 ReadonlyIndex::load_from(&mut index_file, dir, parent_filename, hash_length)?;
             num_parent_commits = parent_file.num_parent_commits + parent_file.num_local_commits;
@@ -1466,7 +1503,6 @@ mod tests {
 
     use super::*;
     use crate::commit_builder::new_change_id;
-    use crate::testutils;
 
     #[test_case(false; "memory")]
     #[test_case(true; "file")]
