@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -13,7 +13,9 @@
 // limitations under the License.
 
 #![deny(unused_must_use)]
-#![cfg_attr(feature = "map_first_last", feature(map_first_last))]
+
+#[macro_use]
+mod content_hash;
 
 pub mod backend;
 pub mod commit;
@@ -28,6 +30,8 @@ pub mod git_backend;
 pub mod gitignore;
 pub mod index;
 pub mod index_store;
+#[cfg(feature = "legacy-thrift")]
+mod legacy_thrift_op_store;
 pub mod local_backend;
 pub mod lock;
 pub mod matchers;
@@ -35,6 +39,7 @@ pub mod nightly_shims;
 pub mod op_heads_store;
 pub mod op_store;
 pub mod operation;
+mod proto_op_store;
 pub mod protos;
 pub mod refs;
 pub mod repo;
@@ -44,9 +49,10 @@ pub mod revset_graph_iterator;
 pub mod rewrite;
 pub mod settings;
 pub mod simple_op_store;
+#[cfg(feature = "legacy-thrift")]
+mod simple_op_store_model;
 pub mod stacked_table;
 pub mod store;
-pub mod testutils;
 pub mod transaction;
 pub mod tree;
 pub mod tree_builder;
