@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,18 +12,18 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::cell::RefCell;
 use std::collections::{HashMap, HashSet};
 use std::fmt::{Debug, Formatter};
-use std::fs;
 use std::io::ErrorKind;
-use std::ops::Deref;
 use std::path::{Path, PathBuf};
-use std::sync::{Arc, Mutex};
+use std::sync::Arc;
+use std::{fs, io};
 
 use itertools::Itertools;
+use once_cell::sync::OnceCell;
 use thiserror::Error;
 
+use self::dirty_cell::DirtyCell;
 use crate::backend::{Backend, BackendError, ChangeId, CommitId};
 use crate::commit::Commit;
 use crate::commit_builder::CommitBuilder;
@@ -43,25 +43,6 @@ use crate::transaction::Transaction;
 use crate::view::{RefName, View};
 use crate::{backend, op_store};
 
-#[derive(Debug, Error, PartialEq, Eq)]
-pub enum RepoError {
-    #[error("Object not found")]
-    NotFound,
-    #[error("Error: {0}")]
-    Other(String),
-}
-
-impl From<BackendError> for RepoError {
-    fn from(err: BackendError) -> Self {
-        match err {
-            BackendError::NotFound => RepoError::NotFound,
-            BackendError::Other(description) => RepoError::Other(description),
-        }
-    }
-}
-
-pub type RepoResult<T> = Result<T, RepoError>;
-
 // TODO: Should we implement From<&ReadonlyRepo> and From<&MutableRepo> for
 // RepoRef?
 #[derive(Clone, Copy)]
@@ -115,7 +96,7 @@ pub struct ReadonlyRepo {
     operation: Operation,
     settings: RepoSettings,
     index_store: Arc<IndexStore>,
-    index: Mutex<Option<Arc<ReadonlyIndex>>>,
+    index: OnceCell<Arc<ReadonlyIndex>>,
     view: View,
 }
 
@@ -129,37 +110,44 @@ impl Debug for ReadonlyRepo {
 }
 
 impl ReadonlyRepo {
-    fn init_repo_dir(repo_path: &Path) {
-        fs::create_dir(repo_path.join("store")).unwrap();
-        fs::create_dir(repo_path.join("op_store")).unwrap();
-        fs::create_dir(repo_path.join("op_heads")).unwrap();
-        fs::create_dir(repo_path.join("index")).unwrap();
-    }
-
     pub fn init(
         user_settings: &UserSettings,
         repo_path: &Path,
         backend_factory: impl FnOnce(&Path) -> Box<dyn Backend>,
-    ) -> Arc<ReadonlyRepo> {
-        let repo_path = repo_path.canonicalize().unwrap();
-        ReadonlyRepo::init_repo_dir(&repo_path);
+    ) -> Result<Arc<ReadonlyRepo>, PathError> {
+        let repo_path = repo_path.canonicalize().context(repo_path)?;
+
         let store_path = repo_path.join("store");
+        fs::create_dir(&store_path).context(&store_path)?;
         let backend = backend_factory(&store_path);
-        fs::write(&store_path.join("backend"), backend.name()).unwrap();
+        let backend_path = store_path.join("backend");
+        fs::write(&backend_path, backend.name()).context(&backend_path)?;
         let store = Store::new(backend);
         let repo_settings = user_settings.with_repo(&repo_path).unwrap();
-        let op_store: Arc<dyn OpStore> = Arc::new(SimpleOpStore::init(repo_path.join("op_store")));
+
+        let op_store_path = repo_path.join("op_store");
+        fs::create_dir(&op_store_path).context(&op_store_path)?;
+        let op_store: Arc<dyn OpStore> = Arc::new(SimpleOpStore::init(op_store_path));
         let mut root_view = op_store::View::default();
         root_view.head_ids.insert(store.root_commit_id().clone());
         root_view
             .public_head_ids
             .insert(store.root_commit_id().clone());
+
+        let op_heads_path = repo_path.join("op_heads");
+        fs::create_dir(&op_heads_path).context(&op_heads_path)?;
+        let operation_metadata =
+            crate::transaction::create_op_metadata(user_settings, "initialize repo".to_string());
         let (op_heads_store, init_op) =
-            OpHeadsStore::init(repo_path.join("op_heads"), &op_store, &root_view);
+            OpHeadsStore::init(op_heads_path, &op_store, &root_view, operation_metadata);
         let op_heads_store = Arc::new(op_heads_store);
-        let index_store = Arc::new(IndexStore::init(repo_path.join("index")));
+
+        let index_path = repo_path.join("index");
+        fs::create_dir(&index_path).context(&index_path)?;
+        let index_store = Arc::new(IndexStore::init(index_path));
+
         let view = View::new(root_view);
-        Arc::new(ReadonlyRepo {
+        Ok(Arc::new(ReadonlyRepo {
             repo_path,
             store,
             op_store,
@@ -167,9 +155,9 @@ impl ReadonlyRepo {
             operation: init_op,
             settings: repo_settings,
             index_store,
-            index: Mutex::new(None),
+            index: OnceCell::new(),
             view,
-        })
+        }))
     }
 
     pub fn load_at_head(
@@ -214,28 +202,10 @@ impl ReadonlyRepo {
     }
 
     pub fn index(&self) -> &Arc<ReadonlyIndex> {
-        let mut locked_index = self.index.lock().unwrap();
-        if locked_index.is_none() {
-            locked_index.replace(
-                self.index_store
-                    .get_index_at_op(&self.operation, &self.store),
-            );
-        }
-        let index: &Arc<ReadonlyIndex> = locked_index.as_ref().unwrap();
-        // Extend lifetime from that of mutex lock to that of self. Safe since we never
-        // change value once it's been set (except in `reindex()` but that
-        // requires a mutable reference).
-        let index: &Arc<ReadonlyIndex> = unsafe { std::mem::transmute(index) };
-        index
-    }
-
-    pub fn reindex(&mut self) -> &Arc<ReadonlyIndex> {
-        self.index_store.reinit();
-        {
-            let mut locked_index = self.index.lock().unwrap();
-            locked_index.take();
-        }
-        self.index()
+        self.index.get_or_init(|| {
+            self.index_store
+                .get_index_at_op(&self.operation, &self.store)
+        })
     }
 
     pub fn store(&self) -> &Arc<Store> {
@@ -258,9 +228,13 @@ impl ReadonlyRepo {
         &self.settings
     }
 
-    pub fn start_transaction(self: &Arc<ReadonlyRepo>, description: &str) -> Transaction {
+    pub fn start_transaction(
+        self: &Arc<ReadonlyRepo>,
+        user_settings: &UserSettings,
+        description: &str,
+    ) -> Transaction {
         let mut_repo = MutableRepo::new(self.clone(), self.index().clone(), &self.view);
-        Transaction::new(mut_repo, description)
+        Transaction::new(mut_repo, user_settings, description)
     }
 
     pub fn reload_at_head(
@@ -298,7 +272,7 @@ pub struct UnresolvedHeadRepo {
 impl UnresolvedHeadRepo {
     pub fn resolve(self, user_settings: &UserSettings) -> Result<Arc<ReadonlyRepo>, BackendError> {
         let base_repo = self.repo_loader.load_at(&self.op_heads[0]);
-        let mut tx = base_repo.start_transaction("resolve concurrent operations");
+        let mut tx = base_repo.start_transaction(user_settings, "resolve concurrent operations");
         for other_op_head in self.op_heads.into_iter().skip(1) {
             tx.merge_operation(other_op_head);
             tx.mut_repo().rebase_descendants(user_settings)?;
@@ -451,7 +425,7 @@ impl RepoLoader {
             operation,
             settings: self.repo_settings.clone(),
             index_store: self.index_store.clone(),
-            index: Mutex::new(Some(index)),
+            index: OnceCell::with_value(index),
             view,
         };
         Arc::new(repo)
@@ -466,7 +440,7 @@ impl RepoLoader {
             operation,
             settings: self.repo_settings.clone(),
             index_store: self.index_store.clone(),
-            index: Mutex::new(None),
+            index: OnceCell::new(),
             view,
         };
         Arc::new(repo)
@@ -476,8 +450,7 @@ impl RepoLoader {
 pub struct MutableRepo {
     base_repo: Arc<ReadonlyRepo>,
     index: MutableIndex,
-    view: RefCell<View>,
-    view_dirty: bool,
+    view: DirtyCell<View>,
     rewritten_commits: HashMap<CommitId, HashSet<CommitId>>,
     abandoned_commits: HashSet<CommitId>,
 }
@@ -493,8 +466,7 @@ impl MutableRepo {
         MutableRepo {
             base_repo,
             index: mut_index,
-            view: RefCell::new(mut_view),
-            view_dirty: false,
+            view: DirtyCell::with_clean(mut_view),
             rewritten_commits: Default::default(),
             abandoned_commits: Default::default(),
         }
@@ -521,10 +493,8 @@ impl MutableRepo {
     }
 
     pub fn view(&self) -> &View {
-        self.enforce_view_invariants();
-        let view_borrow = self.view.borrow();
-        let view = view_borrow.deref();
-        unsafe { std::mem::transmute(view) }
+        self.view
+            .get_or_ensure_clean(|v| self.enforce_view_invariants(v))
     }
 
     fn view_mut(&mut self) -> &mut View {
@@ -532,12 +502,11 @@ impl MutableRepo {
     }
 
     pub fn has_changes(&self) -> bool {
-        self.enforce_view_invariants();
-        self.view.borrow().deref() != &self.base_repo.view
+        self.view() != &self.base_repo.view
     }
 
     pub fn consume(self) -> (MutableIndex, View) {
-        self.enforce_view_invariants();
+        self.view.ensure_clean(|v| self.enforce_view_invariants(v));
         (self.index, self.view.into_inner())
     }
 
@@ -554,6 +523,7 @@ impl MutableRepo {
     /// convenient place to record it. It won't matter after the transaction
     /// has been committed.
     pub fn record_rewritten_commit(&mut self, old_id: CommitId, new_id: CommitId) {
+        assert_ne!(old_id, *self.store().root_commit_id());
         self.rewritten_commits
             .entry(old_id)
             .or_default()
@@ -571,6 +541,7 @@ impl MutableRepo {
     /// convenient place to record it. It won't matter after the transaction
     /// has been committed.
     pub fn record_abandoned_commit(&mut self, old_id: CommitId) {
+        assert_ne!(old_id, *self.store().root_commit_id());
         self.abandoned_commits.insert(old_id);
     }
 
@@ -606,8 +577,16 @@ impl MutableRepo {
         Ok(rebaser.rebased().len())
     }
 
-    pub fn set_wc_commit(&mut self, workspace_id: WorkspaceId, commit_id: CommitId) {
+    pub fn set_wc_commit(
+        &mut self,
+        workspace_id: WorkspaceId,
+        commit_id: CommitId,
+    ) -> Result<(), RewriteRootCommit> {
+        if &commit_id == self.store().root_commit_id() {
+            return Err(RewriteRootCommit);
+        }
         self.view_mut().set_wc_commit(workspace_id, commit_id);
+        Ok(())
     }
 
     pub fn remove_wc_commit(&mut self, workspace_id: &WorkspaceId) {
@@ -621,20 +600,30 @@ impl MutableRepo {
         commit: &Commit,
     ) -> Commit {
         self.leave_commit(&workspace_id);
-        let open_commit =
-            CommitBuilder::for_open_commit(settings, commit.id().clone(), commit.tree_id().clone())
-                .write_to_repo(self);
-        self.set_wc_commit(workspace_id, open_commit.id().clone());
-        open_commit
+        let wc_commit = CommitBuilder::for_new_commit(
+            settings,
+            vec![commit.id().clone()],
+            commit.tree_id().clone(),
+        )
+        .write_to_repo(self);
+        self.set_wc_commit(workspace_id, wc_commit.id().clone())
+            .unwrap();
+        wc_commit
     }
 
-    pub fn edit(&mut self, workspace_id: WorkspaceId, commit: &Commit) {
+    pub fn edit(
+        &mut self,
+        workspace_id: WorkspaceId,
+        commit: &Commit,
+    ) -> Result<(), RewriteRootCommit> {
         self.leave_commit(&workspace_id);
-        self.set_wc_commit(workspace_id, commit.id().clone());
+        self.set_wc_commit(workspace_id, commit.id().clone())
     }
 
     fn leave_commit(&mut self, workspace_id: &WorkspaceId) {
-        let maybe_wc_commit_id = self.view.borrow().get_wc_commit_id(workspace_id).cloned();
+        let maybe_wc_commit_id = self
+            .view
+            .with_ref(|v| v.get_wc_commit_id(workspace_id).cloned());
         if let Some(wc_commit_id) = maybe_wc_commit_id {
             let wc_commit = self.store().get_commit(&wc_commit_id).unwrap();
             if wc_commit.is_empty()
@@ -647,12 +636,8 @@ impl MutableRepo {
         }
     }
 
-    fn enforce_view_invariants(&self) {
-        if !self.view_dirty {
-            return;
-        }
-        let mut view_borrow_mut = self.view.borrow_mut();
-        let view = view_borrow_mut.store_view_mut();
+    fn enforce_view_invariants(&self, view: &mut View) {
+        let view = view.store_view_mut();
         view.public_head_ids = self
             .index
             .heads(view.public_head_ids.iter())
@@ -699,27 +684,27 @@ impl MutableRepo {
                 self.index.add_commit(missing_commit);
             }
             self.view.get_mut().add_head(head.id());
-            self.view_dirty = true;
+            self.view.mark_dirty();
         }
     }
 
     pub fn remove_head(&mut self, head: &CommitId) {
         self.view_mut().remove_head(head);
-        self.view_dirty = true;
+        self.view.mark_dirty();
     }
 
     pub fn add_public_head(&mut self, head: &Commit) {
         self.view_mut().add_public_head(head.id());
-        self.view_dirty = true;
+        self.view.mark_dirty();
     }
 
     pub fn remove_public_head(&mut self, head: &CommitId) {
         self.view_mut().remove_public_head(head);
-        self.view_dirty = true;
+        self.view.mark_dirty();
     }
 
     pub fn get_branch(&self, name: &str) -> Option<BranchTarget> {
-        self.view.borrow().get_branch(name).cloned()
+        self.view.with_ref(|v| v.get_branch(name).cloned())
     }
 
     pub fn set_branch(&mut self, name: String, target: BranchTarget) {
@@ -731,7 +716,7 @@ impl MutableRepo {
     }
 
     pub fn get_local_branch(&self, name: &str) -> Option<RefTarget> {
-        self.view.borrow().get_local_branch(name)
+        self.view.with_ref(|v| v.get_local_branch(name))
     }
 
     pub fn set_local_branch(&mut self, name: String, target: RefTarget) {
@@ -743,7 +728,8 @@ impl MutableRepo {
     }
 
     pub fn get_remote_branch(&self, name: &str, remote_name: &str) -> Option<RefTarget> {
-        self.view.borrow().get_remote_branch(name, remote_name)
+        self.view
+            .with_ref(|v| v.get_remote_branch(name, remote_name))
     }
 
     pub fn set_remote_branch(&mut self, name: String, remote_name: String, target: RefTarget) {
@@ -754,8 +740,12 @@ impl MutableRepo {
         self.view_mut().remove_remote_branch(name, remote_name);
     }
 
+    pub fn rename_remote(&mut self, old: &str, new: &str) {
+        self.view_mut().rename_remote(old, new);
+    }
+
     pub fn get_tag(&self, name: &str) -> Option<RefTarget> {
-        self.view.borrow().get_tag(name)
+        self.view.with_ref(|v| v.get_tag(name))
     }
 
     pub fn set_tag(&mut self, name: String, target: RefTarget) {
@@ -766,6 +756,10 @@ impl MutableRepo {
         self.view_mut().remove_tag(name);
     }
 
+    pub fn get_git_ref(&self, name: &str) -> Option<RefTarget> {
+        self.view.with_ref(|v| v.get_git_ref(name))
+    }
+
     pub fn set_git_ref(&mut self, name: String, target: RefTarget) {
         self.view_mut().set_git_ref(name, target);
     }
@@ -784,7 +778,7 @@ impl MutableRepo {
 
     pub fn set_view(&mut self, data: op_store::View) {
         self.view_mut().set_view(data);
-        self.view_dirty = true;
+        self.view.mark_dirty();
     }
 
     pub fn merge(&mut self, base_repo: &ReadonlyRepo, other_repo: &ReadonlyRepo) {
@@ -795,9 +789,9 @@ impl MutableRepo {
         self.index.merge_in(base_repo.index());
         self.index.merge_in(other_repo.index());
 
-        self.enforce_view_invariants();
+        self.view.ensure_clean(|v| self.enforce_view_invariants(v));
         self.merge_view(&base_repo.view, &other_repo.view);
-        self.view_dirty = true;
+        self.view.mark_dirty();
     }
 
     fn merge_view(&mut self, base: &View, other: &View) {
@@ -961,3 +955,81 @@ impl MutableRepo {
         );
     }
 }
+
+/// Error from attempts to check out the root commit for editing
+#[derive(Debug, Copy, Clone, Error)]
+#[error("Cannot rewrite the root commit")]
+pub struct RewriteRootCommit;
+
+#[derive(Debug, Error)]
+#[error("Cannot access {path}")]
+pub struct PathError {
+    pub path: PathBuf,
+    #[source]
+    pub error: io::Error,
+}
+
+pub(crate) trait IoResultExt<T> {
+    fn context(self, path: impl AsRef<Path>) -> Result<T, PathError>;
+}
+
+impl<T> IoResultExt<T> for io::Result<T> {
+    fn context(self, path: impl AsRef<Path>) -> Result<T, PathError> {
+        self.map_err(|error| PathError {
+            path: path.as_ref().to_path_buf(),
+            error,
+        })
+    }
+}
+
+mod dirty_cell {
+    use std::cell::{Cell, RefCell};
+
+    /// Cell that lazily updates the value after `mark_dirty()`.
+    #[derive(Clone, Debug)]
+    pub struct DirtyCell<T> {
+        value: RefCell<T>,
+        dirty: Cell<bool>,
+    }
+
+    impl<T> DirtyCell<T> {
+        pub fn with_clean(value: T) -> Self {
+            DirtyCell {
+                value: RefCell::new(value),
+                dirty: Cell::new(false),
+            }
+        }
+
+        pub fn get_or_ensure_clean(&self, f: impl FnOnce(&mut T)) -> &T {
+            // SAFETY: get_mut/mark_dirty(&mut self) should invalidate any previously-clean
+            // references leaked by this method. Clean value never changes until then.
+            self.ensure_clean(f);
+            unsafe { &*self.value.as_ptr() }
+        }
+
+        pub fn ensure_clean(&self, f: impl FnOnce(&mut T)) {
+            if self.dirty.get() {
+                // This borrow_mut() ensures that there is no dirty temporary reference.
+                // Panics if ensure_clean() is invoked from with_ref() callback for example.
+                f(&mut self.value.borrow_mut());
+                self.dirty.set(false);
+            }
+        }
+
+        pub fn into_inner(self) -> T {
+            self.value.into_inner()
+        }
+
+        pub fn with_ref<R>(&self, f: impl FnOnce(&T) -> R) -> R {
+            f(&self.value.borrow())
+        }
+
+        pub fn get_mut(&mut self) -> &mut T {
+            self.value.get_mut()
+        }
+
+        pub fn mark_dirty(&mut self) {
+            *self.dirty.get_mut() = true;
+        }
+    }
+}
