@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -145,6 +145,9 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         rewritten: HashMap<CommitId, HashSet<CommitId>>,
         abandoned: HashSet<CommitId>,
     ) -> DescendantRebaser<'settings, 'repo> {
+        let root_commit_id = mut_repo.store().root_commit_id();
+        assert!(!abandoned.contains(root_commit_id));
+        assert!(!rewritten.contains_key(root_commit_id));
         let old_commits_expression = RevsetExpression::commits(rewritten.keys().cloned().collect())
             .union(&RevsetExpression::commits(
                 abandoned.iter().cloned().collect(),
@@ -348,15 +351,15 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         let new_wc_commit = if edit {
             new_commit
         } else {
-            CommitBuilder::for_open_commit(
+            CommitBuilder::for_new_commit(
                 self.settings,
-                new_commit.id().clone(),
+                vec![new_commit.id().clone()],
                 new_commit.tree_id().clone(),
             )
             .write_to_repo(self.mut_repo)
         };
         for workspace_id in workspaces_to_update.into_iter() {
-            self.mut_repo.edit(workspace_id, &new_wc_commit);
+            self.mut_repo.edit(workspace_id, &new_wc_commit).unwrap();
         }
         Ok(())
     }
