@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -29,18 +29,37 @@ pub struct RepoSettings {
     _config: config::Config,
 }
 
+fn get_timestamp_config(config: &config::Config, key: &str) -> Option<Timestamp> {
+    match config.get_string(key) {
+        Ok(timestamp_str) => match DateTime::parse_from_rfc3339(&timestamp_str) {
+            Ok(datetime) => Some(Timestamp::from_datetime(datetime)),
+            Err(_) => None,
+        },
+        Err(_) => None,
+    }
+}
+
 impl UserSettings {
     pub fn from_config(config: config::Config) -> Self {
-        let timestamp = match config.get_string("user.timestamp") {
-            Ok(timestamp_str) => match DateTime::parse_from_rfc3339(&timestamp_str) {
-                Ok(datetime) => Some(Timestamp::from_datetime(datetime)),
-                Err(_) => None,
-            },
-            Err(_) => None,
-        };
+        let timestamp = get_timestamp_config(&config, "user.timestamp");
         UserSettings { config, timestamp }
     }
 
+    pub fn with_toml_strings(
+        &self,
+        toml_strs: &[String],
+    ) -> Result<UserSettings, config::ConfigError> {
+        let mut config_builder = config::Config::builder().add_source(self.config.clone());
+        for s in toml_strs {
+            config_builder =
+                config_builder.add_source(config::File::from_str(s, config::FileFormat::Toml));
+        }
+        Ok(UserSettings {
+            config: config_builder.build()?,
+            timestamp: self.timestamp.clone(),
+        })
+    }
+
     pub fn with_repo(&self, repo_path: &Path) -> Result<RepoSettings, config::ConfigError> {
         let config = config::Config::builder()
             .add_source(self.config.clone())
@@ -73,6 +92,22 @@ impl UserSettings {
         "(no email configured)"
     }
 
+    pub fn operation_timestamp(&self) -> Option<Timestamp> {
+        get_timestamp_config(&self.config, "operation.timestamp")
+    }
+
+    pub fn operation_hostname(&self) -> String {
+        self.config
+            .get_string("operation.hostname")
+            .unwrap_or_else(|_| whoami::hostname())
+    }
+
+    pub fn operation_username(&self) -> String {
+        self.config
+            .get_string("operation.username")
+            .unwrap_or_else(|_| whoami::username())
+    }
+
     pub fn push_branch_prefix(&self) -> String {
         self.config
             .get_string("push.branch-prefix")
@@ -82,7 +117,9 @@ impl UserSettings {
     pub fn default_revset(&self) -> String {
         self.config
             .get_string("ui.default-revset")
-            .unwrap_or_else(|_| "remote_branches().. | (remote_branches()..)-".to_string())
+            .unwrap_or_else(|_| {
+                "@ | (remote_branches() | tags()).. | ((remote_branches() | tags())..)-".to_string()
+            })
     }
 
     pub fn signature(&self) -> Signature {
@@ -94,18 +131,18 @@ impl UserSettings {
         }
     }
 
-    pub fn enable_open_commits(&self) -> bool {
-        self.config
-            .get_bool("ui.enable-open-commits")
-            .unwrap_or(false)
-    }
-
     pub fn allow_native_backend(&self) -> bool {
         self.config
             .get_bool("ui.allow-init-native")
             .unwrap_or(false)
     }
 
+    pub fn relative_timestamps(&self) -> bool {
+        self.config
+            .get_bool("ui.relative-timestamps")
+            .unwrap_or(false)
+    }
+
     pub fn config(&self) -> &config::Config {
         &self.config
     }
