@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,7 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::collections::HashMap;
 use std::sync::Arc;
 
 use crate::backend::Timestamp;
@@ -22,25 +21,30 @@ use crate::op_store;
 use crate::op_store::OperationMetadata;
 use crate::operation::Operation;
 use crate::repo::{MutableRepo, ReadonlyRepo, RepoLoader};
+use crate::settings::UserSettings;
 use crate::view::View;
 
 pub struct Transaction {
     repo: Option<MutableRepo>,
     parent_ops: Vec<Operation>,
-    description: String,
-    start_time: Timestamp,
-    tags: HashMap<String, String>,
+    op_metadata: OperationMetadata,
+    end_time: Option<Timestamp>,
 }
 
 impl Transaction {
-    pub fn new(mut_repo: MutableRepo, description: &str) -> Transaction {
+    pub fn new(
+        mut_repo: MutableRepo,
+        user_settings: &UserSettings,
+        description: &str,
+    ) -> Transaction {
         let parent_ops = vec![mut_repo.base_repo().operation().clone()];
+        let op_metadata = create_op_metadata(user_settings, description.to_string());
+        let end_time = user_settings.operation_timestamp();
         Transaction {
             repo: Some(mut_repo),
             parent_ops,
-            description: description.to_owned(),
-            start_time: Timestamp::now(),
-            tags: Default::default(),
+            op_metadata,
+            end_time,
         }
     }
 
@@ -49,7 +53,7 @@ impl Transaction {
     }
 
     pub fn set_tag(&mut self, key: String, value: String) {
-        self.tags.insert(key, value);
+        self.op_metadata.tags.insert(key, value);
     }
 
     pub fn repo(&self) -> &MutableRepo {
@@ -96,14 +100,12 @@ impl Transaction {
         let index = base_repo.index_store().write_index(mut_index).unwrap();
 
         let view_id = base_repo.op_store().write_view(view.store_view()).unwrap();
-        let mut operation_metadata =
-            OperationMetadata::new(self.description.clone(), self.start_time.clone());
-        operation_metadata.tags = self.tags.clone();
+        self.op_metadata.end_time = self.end_time.unwrap_or_else(Timestamp::now);
         let parents = self.parent_ops.iter().map(|op| op.id().clone()).collect();
         let store_operation = op_store::Operation {
             view_id,
             parents,
-            metadata: operation_metadata,
+            metadata: self.op_metadata,
         };
         let new_op_id = base_repo
             .op_store()
@@ -119,6 +121,23 @@ impl Transaction {
     }
 }
 
+pub fn create_op_metadata(user_settings: &UserSettings, description: String) -> OperationMetadata {
+    let start_time = user_settings
+        .operation_timestamp()
+        .unwrap_or_else(Timestamp::now);
+    let end_time = start_time.clone();
+    let hostname = user_settings.operation_hostname();
+    let username = user_settings.operation_username();
+    OperationMetadata {
+        start_time,
+        end_time,
+        description,
+        hostname,
+        username,
+        tags: Default::default(),
+    }
+}
+
 struct NewRepoData {
     operation: Operation,
     view: View,
