@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -210,19 +210,23 @@ impl Tree {
         }
     }
 
-    pub fn has_conflict(&self) -> bool {
-        !self.conflicts().is_empty()
-    }
-
-    pub fn conflicts(&self) -> Vec<(RepoPath, ConflictId)> {
+    pub fn conflicts_matching(&self, matcher: &dyn Matcher) -> Vec<(RepoPath, ConflictId)> {
         let mut conflicts = vec![];
-        for (name, value) in self.entries() {
+        for (name, value) in self.entries_matching(matcher) {
             if let TreeValue::Conflict(id) = value {
                 conflicts.push((name.clone(), id.clone()));
             }
         }
         conflicts
     }
+
+    pub fn conflicts(&self) -> Vec<(RepoPath, ConflictId)> {
+        self.conflicts_matching(&EverythingMatcher)
+    }
+
+    pub fn has_conflict(&self) -> bool {
+        !self.conflicts().is_empty()
+    }
 }
 
 pub struct TreeEntriesIterator<'matcher> {
@@ -633,7 +637,7 @@ fn merge_tree_value(
                 try_resolve_file_conflict(store, &filename, &conflict)?
             {
                 let id = store.write_file(&filename, &mut merged_content.as_slice())?;
-                Some(TreeValue::Normal { id, executable })
+                Some(TreeValue::File { id, executable })
             } else {
                 let conflict_id = store.write_conflict(&filename, &conflict)?;
                 Some(TreeValue::Conflict(conflict_id))
@@ -657,7 +661,7 @@ fn try_resolve_file_conflict(
     let mut added_file_ids = vec![];
     for part in &conflict.removes {
         match &part.value {
-            TreeValue::Normal { id, executable } => {
+            TreeValue::File { id, executable } => {
                 if *executable {
                     exec_delta -= 1;
                 } else {
@@ -672,7 +676,7 @@ fn try_resolve_file_conflict(
     }
     for part in &conflict.adds {
         match &part.value {
-            TreeValue::Normal { id, executable } => {
+            TreeValue::File { id, executable } => {
                 if *executable {
                     exec_delta += 1;
                 } else {
