@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -48,6 +48,7 @@ use crate::tree_builder::TreeBuilder;
 pub enum FileType {
     Normal { executable: bool },
     Symlink,
+    GitSubmodule,
     Conflict { id: ConflictId },
 }
 
@@ -89,6 +90,14 @@ impl FileState {
         }
     }
 
+    fn for_gitsubmodule() -> Self {
+        FileState {
+            file_type: FileType::GitSubmodule,
+            mtime: MillisSinceEpoch(0),
+            size: 0,
+        }
+    }
+
     #[cfg_attr(unix, allow(dead_code))]
     fn is_executable(&self) -> bool {
         if let FileType::Normal { executable } = &self.file_type {
@@ -125,6 +134,7 @@ fn file_state_from_proto(proto: &crate::protos::working_copy::FileState) -> File
             let id = ConflictId::new(proto.conflict_id.to_vec());
             FileType::Conflict { id }
         }
+        crate::protos::working_copy::FileType::GitSubmodule => FileType::GitSubmodule,
     };
     FileState {
         file_type,
@@ -143,6 +153,7 @@ fn file_state_to_proto(file_state: &FileState) -> crate::protos::working_copy::F
             proto.conflict_id = id.to_bytes();
             crate::protos::working_copy::FileType::Conflict
         }
+        FileType::GitSubmodule => crate::protos::working_copy::FileType::GitSubmodule,
     };
     proto.file_type = EnumOrUnknown::new(file_type);
     proto.mtime_millis_since_epoch = file_state.mtime.0;
@@ -468,8 +479,19 @@ impl TreeState {
             self.working_copy_path.clone(),
             base_ignores,
         )];
+
         let mut tree_builder = self.store.tree_builder(self.tree_id.clone());
-        let mut deleted_files: HashSet<_> = self.file_states.keys().cloned().collect();
+        let mut deleted_files: HashSet<_> = self
+            .file_states
+            .iter()
+            .filter_map(|(path, state)| {
+                if state.file_type != FileType::GitSubmodule {
+                    Some(path.clone())
+                } else {
+                    None
+                }
+            })
+            .collect();
         while let Some((dir, disk_dir, git_ignore)) = work.pop() {
             if sparse_matcher.visit(&dir).is_nothing() {
                 continue;
@@ -489,6 +511,12 @@ impl TreeState {
                     continue;
                 }
                 let sub_path = dir.join(&RepoPathComponent::from(name));
+                if let Some(file_state) = self.file_states.get(&sub_path) {
+                    if file_state.file_type == FileType::GitSubmodule {
+                        continue;
+                    }
+                }
+
                 if file_type.is_dir() {
                     // If the whole directory is ignored, skip it unless we're already tracking
                     // some file in it.
@@ -555,6 +583,7 @@ impl TreeState {
             // ignore it.
             return Ok(());
         }
+
         let disk_path = dir_entry.path();
         let metadata = dir_entry.metadata().map_err(|err| SnapshotError::IoError {
             message: format!("Failed to stat file {}", disk_path.display()),
@@ -654,13 +683,14 @@ impl TreeState {
         match file_type {
             FileType::Normal { executable } => {
                 let id = self.write_file_to_store(repo_path, disk_path)?;
-                Ok(TreeValue::Normal { id, executable })
+                Ok(TreeValue::File { id, executable })
             }
             FileType::Symlink => {
                 let id = self.write_symlink_to_store(repo_path, disk_path)?;
                 Ok(TreeValue::Symlink(id))
             }
             FileType::Conflict { .. } => panic!("conflicts should be handled by the caller"),
+            FileType::GitSubmodule => panic!("git submodule cannot be written to store"),
         }
     }
 
@@ -851,14 +881,14 @@ impl TreeState {
                 }
                 Diff::Added(after) => {
                     let file_state = match after {
-                        TreeValue::Normal { id, executable } => {
+                        TreeValue::File { id, executable } => {
                             self.write_file(&disk_path, &path, &id, executable)?
                         }
                         TreeValue::Symlink(id) => self.write_symlink(&disk_path, &path, &id)?,
                         TreeValue::Conflict(id) => self.write_conflict(&disk_path, &path, &id)?,
                         TreeValue::GitSubmodule(_id) => {
                             println!("ignoring git submodule at {:?}", path);
-                            return Ok(());
+                            FileState::for_gitsubmodule()
                         }
                         TreeValue::Tree(_id) => {
                             panic!("unexpected tree entry in diff at {:?}", path);
@@ -868,11 +898,11 @@ impl TreeState {
                     stats.added_files += 1;
                 }
                 Diff::Modified(
-                    TreeValue::Normal {
+                    TreeValue::File {
                         id: old_id,
                         executable: old_executable,
                     },
-                    TreeValue::Normal { id, executable },
+                    TreeValue::File { id, executable },
                 ) if id == old_id => {
                     // Optimization for when only the executable bit changed
                     assert_ne!(executable, old_executable);
@@ -884,7 +914,7 @@ impl TreeState {
                 Diff::Modified(before, after) => {
                     fs::remove_file(&disk_path).ok();
                     let file_state = match (before, after) {
-                        (_, TreeValue::Normal { id, executable }) => {
+                        (_, TreeValue::File { id, executable }) => {
                             self.write_file(&disk_path, &path, &id, executable)?
                         }
                         (_, TreeValue::Symlink(id)) => {
@@ -895,8 +925,7 @@ impl TreeState {
                         }
                         (_, TreeValue::GitSubmodule(_id)) => {
                             println!("ignoring git submodule at {:?}", path);
-                            self.file_states.remove(&path);
-                            return Ok(());
+                            FileState::for_gitsubmodule()
                         }
                         (_, TreeValue::Tree(_id)) => {
                             panic!("unexpected tree entry in diff at {:?}", path);
@@ -932,12 +961,12 @@ impl TreeState {
                 }
                 Diff::Added(after) | Diff::Modified(_, after) => {
                     let file_type = match after {
-                        TreeValue::Normal { id: _, executable } => FileType::Normal { executable },
+                        TreeValue::File { id: _, executable } => FileType::Normal { executable },
                         TreeValue::Symlink(_id) => FileType::Symlink,
                         TreeValue::Conflict(id) => FileType::Conflict { id },
                         TreeValue::GitSubmodule(_id) => {
                             println!("ignoring git submodule at {:?}", path);
-                            continue;
+                            FileType::GitSubmodule
                         }
                         TreeValue::Tree(_id) => {
                             panic!("unexpected tree entry in diff at {:?}", path);
