@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -16,10 +16,9 @@ use jujutsu_lib::commit_builder::CommitBuilder;
 use jujutsu_lib::matchers::EverythingMatcher;
 use jujutsu_lib::repo_path::RepoPath;
 use jujutsu_lib::settings::UserSettings;
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::{assert_rebased, CommitGraphBuilder, TestRepo};
 use jujutsu_lib::tree::DiffSummary;
 use test_case::test_case;
+use testutils::{assert_rebased, CommitGraphBuilder, TestRepo};
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -39,7 +38,7 @@ fn test_initial(use_git: bool) {
         ],
     );
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit = CommitBuilder::for_new_commit(
         &settings,
         vec![store.root_commit_id().clone()],
@@ -50,7 +49,6 @@ fn test_initial(use_git: bool) {
 
     assert_eq!(commit.parents(), vec![store.root_commit()]);
     assert_eq!(commit.predecessors(), vec![]);
-    assert!(!commit.is_open());
     assert_eq!(commit.description(), "");
     assert_eq!(commit.author().name, settings.user_name());
     assert_eq!(commit.author().email, settings.user_email());
@@ -87,7 +85,7 @@ fn test_rewrite(use_git: bool) {
         ],
     );
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let initial_commit = CommitBuilder::for_new_commit(
         &settings,
         vec![store.root_commit_id().clone()],
@@ -112,7 +110,7 @@ fn test_rewrite(use_git: bool) {
         .build()
         .unwrap();
     let rewrite_settings = UserSettings::from_config(config);
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let rewritten_commit = CommitBuilder::for_rewrite_from(&rewrite_settings, &initial_commit)
         .set_tree(rewritten_tree.id().clone())
         .write_to_repo(tx.mut_repo());
@@ -123,7 +121,6 @@ fn test_rewrite(use_git: bool) {
         rewritten_commit.predecessors(),
         vec![initial_commit.clone()]
     );
-    assert!(!rewritten_commit.is_open());
     assert_eq!(rewritten_commit.author().name, settings.user_name());
     assert_eq!(rewritten_commit.author().email, settings.user_email());
     assert_eq!(
@@ -167,7 +164,7 @@ fn test_rewrite_update_missing_user(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&missing_user_settings, "test");
     let initial_commit = CommitBuilder::for_new_commit(
         &missing_user_settings,
         vec![repo.store().root_commit_id().clone()],
@@ -210,7 +207,7 @@ fn test_commit_builder_descendants(use_git: bool) {
     let repo = &test_repo.repo;
     let store = repo.store().clone();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
@@ -218,7 +215,7 @@ fn test_commit_builder_descendants(use_git: bool) {
     let repo = tx.commit();
 
     // Test with for_new_commit()
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     CommitBuilder::for_new_commit(
         &settings,
         vec![store.root_commit_id().clone()],
@@ -228,26 +225,15 @@ fn test_commit_builder_descendants(use_git: bool) {
     let mut rebaser = tx.mut_repo().create_descendant_rebaser(&settings);
     assert!(rebaser.rebase_next().unwrap().is_none());
 
-    // Test with for_open_commit()
-    let mut tx = repo.start_transaction("test");
-    CommitBuilder::for_open_commit(
-        &settings,
-        commit2.id().clone(),
-        store.empty_tree_id().clone(),
-    )
-    .write_to_repo(tx.mut_repo());
-    let mut rebaser = tx.mut_repo().create_descendant_rebaser(&settings);
-    assert!(rebaser.rebase_next().unwrap().is_none());
-
     // Test with for_rewrite_from()
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit4 = CommitBuilder::for_rewrite_from(&settings, &commit2).write_to_repo(tx.mut_repo());
     let mut rebaser = tx.mut_repo().create_descendant_rebaser(&settings);
     assert_rebased(rebaser.rebase_next().unwrap(), &commit3, &[&commit4]);
     assert!(rebaser.rebase_next().unwrap().is_none());
 
     // Test with for_rewrite_from() but new change id
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     CommitBuilder::for_rewrite_from(&settings, &commit2)
         .generate_new_change_id()
         .write_to_repo(tx.mut_repo());
