@@ -1,4 +1,4 @@
-// Copyright 2021 Google LLC
+// Copyright 2021 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 // limitations under the License.
 
 use jujutsu_lib::repo::{BackendFactories, RepoLoader};
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::TestRepo;
 use test_case::test_case;
+use testutils::{create_random_commit, TestRepo};
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -24,11 +23,11 @@ fn test_load_at_operation(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("add commit");
-    let commit = testutils::create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "add commit");
+    let commit = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("remove commit");
+    let mut tx = repo.start_transaction(&settings, "remove commit");
     tx.mut_repo().remove_head(commit.id());
     tx.commit();
 
