@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -18,10 +18,10 @@ use jujutsu_lib::backend::{ConflictPart, TreeValue};
 use jujutsu_lib::commit_builder::CommitBuilder;
 use jujutsu_lib::repo_path::{RepoPath, RepoPathComponent};
 use jujutsu_lib::rewrite::rebase_commit;
-use jujutsu_lib::testutils::TestRepo;
+use jujutsu_lib::tree;
 use jujutsu_lib::tree::Tree;
-use jujutsu_lib::{testutils, tree};
 use test_case::test_case;
+use testutils::TestRepo;
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -571,7 +571,7 @@ fn test_simplify_conflict_after_resolving_parent(use_git: bool) {
     // rebase C2 (the rebased C) onto the resolved conflict. C3 should not have
     // a conflict since it changed an unrelated line.
     let path = RepoPath::from_internal_string("dir/file");
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let tree_a = testutils::create_tree(repo, &[(&path, "abc\ndef\nghi\n")]);
     let commit_a = CommitBuilder::for_new_commit(
         &settings,
@@ -617,7 +617,7 @@ fn test_simplify_conflict_after_resolving_parent(use_git: bool) {
     // The conflict should now be resolved.
     let resolved_value = commit_c3.tree().path_value(&path);
     match resolved_value {
-        Some(TreeValue::Normal {
+        Some(TreeValue::File {
             id,
             executable: false,
         }) => {
