@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -14,51 +14,42 @@
 
 use jujutsu_lib::commit_builder::CommitBuilder;
 use jujutsu_lib::op_store::{RefTarget, WorkspaceId};
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::{assert_rebased, CommitGraphBuilder, TestRepo};
 use maplit::hashset;
 use test_case::test_case;
-
-// TODO Many of the tests here are not run with Git because they end up creating
-// two commits with the same contents.
+use testutils::{assert_rebased, create_random_commit, CommitGraphBuilder, TestRepo};
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_edit(use_git: bool) {
-    // Test that MutableRepo::check_out() uses the requested commit if it's open
+    // Test that MutableRepo::edit() uses the requested commit (not a new child)
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
-    let wc_commit = testutils::create_random_commit(&settings, repo)
-        .set_open(true)
-        .write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "test");
+    let wc_commit = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let ws_id = WorkspaceId::default();
-    tx.mut_repo().edit(ws_id.clone(), &wc_commit);
+    tx.mut_repo().edit(ws_id.clone(), &wc_commit).unwrap();
     let repo = tx.commit();
     assert_eq!(repo.view().get_wc_commit_id(&ws_id), Some(wc_commit.id()));
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
-fn test_checkout_closed(use_git: bool) {
-    // Test that MutableRepo::check_out() creates a child if the requested commit is
-    // closed
+#[test_case(true ; "git backend")]
+fn test_checkout(use_git: bool) {
+    // Test that MutableRepo::check_out() creates a child
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
-    let requested_checkout = testutils::create_random_commit(&settings, repo)
-        .set_open(false)
-        .write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "test");
+    let requested_checkout = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let ws_id = WorkspaceId::default();
     let actual_checkout = tx
         .mut_repo()
@@ -74,7 +65,7 @@ fn test_checkout_closed(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_checkout_previous_not_empty(use_git: bool) {
     // Test that MutableRepo::check_out() does not usually abandon the previous
     // commit.
@@ -82,27 +73,23 @@ fn test_checkout_previous_not_empty(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let old_checkout = testutils::create_random_commit(&settings, repo)
-        .set_open(true)
-        .write_to_repo(mut_repo);
+    let old_checkout = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     let ws_id = WorkspaceId::default();
-    mut_repo.edit(ws_id.clone(), &old_checkout);
+    mut_repo.edit(ws_id.clone(), &old_checkout).unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let new_checkout = testutils::create_random_commit(&settings, &repo)
-        .set_open(true)
-        .write_to_repo(mut_repo);
-    mut_repo.edit(ws_id, &new_checkout);
+    let new_checkout = create_random_commit(&settings, &repo).write_to_repo(mut_repo);
+    mut_repo.edit(ws_id, &new_checkout).unwrap();
     mut_repo.rebase_descendants(&settings).unwrap();
     assert!(mut_repo.view().heads().contains(old_checkout.id()));
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_checkout_previous_empty(use_git: bool) {
     // Test that MutableRepo::check_out() abandons the previous commit if it was
     // empty.
@@ -110,30 +97,28 @@ fn test_checkout_previous_empty(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let old_checkout = CommitBuilder::for_open_commit(
+    let old_checkout = CommitBuilder::for_new_commit(
         &settings,
-        repo.store().root_commit_id().clone(),
+        vec![repo.store().root_commit_id().clone()],
         repo.store().empty_tree_id().clone(),
     )
     .write_to_repo(mut_repo);
     let ws_id = WorkspaceId::default();
-    mut_repo.edit(ws_id.clone(), &old_checkout);
+    mut_repo.edit(ws_id.clone(), &old_checkout).unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let new_wc_commit = testutils::create_random_commit(&settings, &repo)
-        .set_open(true)
-        .write_to_repo(mut_repo);
-    mut_repo.edit(ws_id, &new_wc_commit);
+    let new_wc_commit = create_random_commit(&settings, &repo).write_to_repo(mut_repo);
+    mut_repo.edit(ws_id, &new_wc_commit).unwrap();
     mut_repo.rebase_descendants(&settings).unwrap();
     assert!(!mut_repo.view().heads().contains(old_checkout.id()));
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_checkout_previous_empty_with_description(use_git: bool) {
     // Test that MutableRepo::check_out() does not abandon the previous commit if it
     // has a non-empty description.
@@ -141,31 +126,29 @@ fn test_checkout_previous_empty_with_description(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let old_checkout = CommitBuilder::for_open_commit(
+    let old_checkout = CommitBuilder::for_new_commit(
         &settings,
-        repo.store().root_commit_id().clone(),
+        vec![repo.store().root_commit_id().clone()],
         repo.store().empty_tree_id().clone(),
     )
     .set_description("not empty".to_string())
     .write_to_repo(mut_repo);
     let ws_id = WorkspaceId::default();
-    mut_repo.edit(ws_id.clone(), &old_checkout);
+    mut_repo.edit(ws_id.clone(), &old_checkout).unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let new_checkout = testutils::create_random_commit(&settings, &repo)
-        .set_open(true)
-        .write_to_repo(mut_repo);
-    mut_repo.edit(ws_id, &new_checkout);
+    let new_checkout = create_random_commit(&settings, &repo).write_to_repo(mut_repo);
+    mut_repo.edit(ws_id, &new_checkout).unwrap();
     mut_repo.rebase_descendants(&settings).unwrap();
     assert!(mut_repo.view().heads().contains(old_checkout.id()));
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_checkout_previous_empty_non_head(use_git: bool) {
     // Test that MutableRepo::check_out() does not abandon the previous commit if it
     // was empty and is not a head
@@ -173,30 +156,28 @@ fn test_checkout_previous_empty_non_head(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let old_checkout = CommitBuilder::for_open_commit(
+    let old_checkout = CommitBuilder::for_new_commit(
         &settings,
-        repo.store().root_commit_id().clone(),
+        vec![repo.store().root_commit_id().clone()],
         repo.store().empty_tree_id().clone(),
     )
     .write_to_repo(mut_repo);
-    let old_child = CommitBuilder::for_open_commit(
+    let old_child = CommitBuilder::for_new_commit(
         &settings,
-        old_checkout.id().clone(),
+        vec![old_checkout.id().clone()],
         old_checkout.tree_id().clone(),
     )
     .write_to_repo(mut_repo);
     let ws_id = WorkspaceId::default();
-    mut_repo.edit(ws_id.clone(), &old_checkout);
+    mut_repo.edit(ws_id.clone(), &old_checkout).unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let new_checkout = testutils::create_random_commit(&settings, &repo)
-        .set_open(true)
-        .write_to_repo(mut_repo);
-    mut_repo.edit(ws_id, &new_checkout);
+    let new_checkout = create_random_commit(&settings, &repo).write_to_repo(mut_repo);
+    mut_repo.edit(ws_id, &new_checkout).unwrap();
     mut_repo.rebase_descendants(&settings).unwrap();
     assert_eq!(
         *mut_repo.view().heads(),
@@ -205,7 +186,7 @@ fn test_checkout_previous_empty_non_head(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_edit_initial(use_git: bool) {
     // Test that MutableRepo::edit() can be used on the initial checkout in a
     // workspace
@@ -213,15 +194,13 @@ fn test_edit_initial(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
-    let checkout = testutils::create_random_commit(&settings, repo)
-        .set_open(true)
-        .write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "test");
+    let checkout = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let workspace_id = WorkspaceId::new("new-workspace".to_string());
-    tx.mut_repo().edit(workspace_id.clone(), &checkout);
+    tx.mut_repo().edit(workspace_id.clone(), &checkout).unwrap();
     let repo = tx.commit();
     assert_eq!(
         repo.view().get_wc_commit_id(&workspace_id),
@@ -230,7 +209,7 @@ fn test_edit_initial(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_add_head_success(use_git: bool) {
     // Test that MutableRepo::add_head() adds the head, and that it's still there
     // after commit. It should also be indexed.
@@ -240,15 +219,15 @@ fn test_add_head_success(use_git: bool) {
 
     // Create a commit outside of the repo by using a temporary transaction. Then
     // add that as a head.
-    let mut tx = repo.start_transaction("test");
-    let new_commit = testutils::create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "test");
+    let new_commit = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     drop(tx);
 
     let index_stats = repo.index().stats();
     assert_eq!(index_stats.num_heads, 1);
     assert_eq!(index_stats.num_commits, 1);
     assert_eq!(index_stats.max_generation_number, 0);
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     assert!(!mut_repo.view().heads().contains(new_commit.id()));
     assert!(!mut_repo.index().has_id(new_commit.id()));
@@ -265,7 +244,7 @@ fn test_add_head_success(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_add_head_ancestor(use_git: bool) {
     // Test that MutableRepo::add_head() does not add a head if it's an ancestor of
     // an existing head.
@@ -273,7 +252,7 @@ fn test_add_head_ancestor(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
@@ -284,7 +263,7 @@ fn test_add_head_ancestor(use_git: bool) {
     assert_eq!(index_stats.num_heads, 1);
     assert_eq!(index_stats.num_commits, 4);
     assert_eq!(index_stats.max_generation_number, 3);
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     mut_repo.add_head(&commit1);
     assert!(!mut_repo.view().heads().contains(commit1.id()));
@@ -295,7 +274,7 @@ fn test_add_head_ancestor(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_add_head_not_immediate_child(use_git: bool) {
     // Test that MutableRepo::add_head() can be used for adding a head that is not
     // an immediate child of a current head.
@@ -303,18 +282,18 @@ fn test_add_head_not_immediate_child(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
-    let initial = testutils::create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "test");
+    let initial = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
     // Create some commit outside of the repo by using a temporary transaction. Then
     // add one of them as a head.
-    let mut tx = repo.start_transaction("test");
-    let rewritten = testutils::create_random_commit(&settings, &repo)
+    let mut tx = repo.start_transaction(&settings, "test");
+    let rewritten = create_random_commit(&settings, &repo)
         .set_change_id(initial.change_id().clone())
         .set_predecessors(vec![initial.id().clone()])
         .write_to_repo(tx.mut_repo());
-    let child = testutils::create_random_commit(&settings, &repo)
+    let child = create_random_commit(&settings, &repo)
         .set_parents(vec![rewritten.id().clone()])
         .write_to_repo(tx.mut_repo());
     drop(tx);
@@ -323,7 +302,7 @@ fn test_add_head_not_immediate_child(use_git: bool) {
     assert_eq!(index_stats.num_heads, 1);
     assert_eq!(index_stats.num_commits, 2);
     assert_eq!(index_stats.max_generation_number, 1);
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     mut_repo.add_head(&child);
     assert!(mut_repo.view().heads().contains(initial.id()));
@@ -339,7 +318,7 @@ fn test_add_head_not_immediate_child(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_remove_head(use_git: bool) {
     // Test that MutableRepo::remove_head() removes the head, and that it's still
     // removed after commit. It should remain in the index, since we otherwise would
@@ -352,14 +331,14 @@ fn test_remove_head(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     let commit3 = graph_builder.commit_with_parents(&[&commit2]);
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     assert!(mut_repo.view().heads().contains(commit3.id()));
     mut_repo.remove_head(commit3.id());
@@ -381,7 +360,7 @@ fn test_remove_head(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_add_public_head(use_git: bool) {
     // Test that MutableRepo::add_public_head() adds the head, and that it's still
     // there after commit.
@@ -389,11 +368,11 @@ fn test_add_public_head(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
+    let mut tx = repo.start_transaction(&settings, "test");
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     assert!(!mut_repo.view().public_heads().contains(commit1.id()));
     mut_repo.add_public_head(&commit1);
@@ -403,7 +382,7 @@ fn test_add_public_head(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_add_public_head_ancestor(use_git: bool) {
     // Test that MutableRepo::add_public_head() does not add a public head if it's
     // an ancestor of an existing public head.
@@ -411,14 +390,14 @@ fn test_add_public_head_ancestor(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     tx.mut_repo().add_public_head(&commit2);
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     assert!(!mut_repo.view().public_heads().contains(commit1.id()));
     mut_repo.add_public_head(&commit1);
@@ -428,7 +407,7 @@ fn test_add_public_head_ancestor(use_git: bool) {
 }
 
 #[test_case(false ; "local backend")]
-// #[test_case(true ; "git backend")]
+#[test_case(true ; "git backend")]
 fn test_remove_public_head(use_git: bool) {
     // Test that MutableRepo::remove_public_head() removes the head, and that it's
     // still removed after commit.
@@ -436,13 +415,13 @@ fn test_remove_public_head(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.add_public_head(&commit1);
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     assert!(mut_repo.view().public_heads().contains(commit1.id()));
     mut_repo.remove_public_head(commit1.id());
@@ -461,14 +440,16 @@ fn test_has_changed(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.remove_head(commit2.id());
     mut_repo.add_public_head(&commit1);
     let ws_id = WorkspaceId::default();
-    mut_repo.set_wc_commit(ws_id.clone(), commit1.id().clone());
+    mut_repo
+        .set_wc_commit(ws_id.clone(), commit1.id().clone())
+        .unwrap();
     mut_repo.set_local_branch("main".to_string(), RefTarget::Normal(commit1.id().clone()));
     mut_repo.set_remote_branch(
         "main".to_string(),
@@ -480,12 +461,14 @@ fn test_has_changed(use_git: bool) {
     assert_eq!(repo.view().heads(), &hashset! {commit1.id().clone()});
     assert_eq!(repo.view().public_heads(), &hashset! {commit1.id().clone()});
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     mut_repo.add_public_head(&commit1);
     mut_repo.add_head(&commit1);
-    mut_repo.set_wc_commit(ws_id.clone(), commit1.id().clone());
+    mut_repo
+        .set_wc_commit(ws_id.clone(), commit1.id().clone())
+        .unwrap();
     mut_repo.set_local_branch("main".to_string(), RefTarget::Normal(commit1.id().clone()));
     mut_repo.set_remote_branch(
         "main".to_string(),
@@ -515,9 +498,11 @@ fn test_has_changed(use_git: bool) {
     mut_repo.remove_head(commit2.id());
     assert!(!mut_repo.has_changes());
 
-    mut_repo.set_wc_commit(ws_id.clone(), commit2.id().clone());
+    mut_repo
+        .set_wc_commit(ws_id.clone(), commit2.id().clone())
+        .unwrap();
     assert!(mut_repo.has_changes());
-    mut_repo.set_wc_commit(ws_id, commit1.id().clone());
+    mut_repo.set_wc_commit(ws_id, commit1.id().clone()).unwrap();
     assert!(!mut_repo.has_changes());
 
     mut_repo.set_local_branch("main".to_string(), RefTarget::Normal(commit2.id().clone()));
@@ -549,7 +534,7 @@ fn test_rebase_descendants_simple(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
@@ -558,7 +543,7 @@ fn test_rebase_descendants_simple(use_git: bool) {
     let commit5 = graph_builder.commit_with_parents(&[&commit4]);
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit6 = graph_builder.commit_with_parents(&[&commit1]);
@@ -587,14 +572,14 @@ fn test_rebase_descendants_conflicting_rewrite(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     let _commit3 = graph_builder.commit_with_parents(&[&commit2]);
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit4 = graph_builder.commit_with_parents(&[&commit1]);
@@ -612,3 +597,19 @@ fn test_rebase_descendants_conflicting_rewrite(use_git: bool) {
         .unwrap()
         .is_none());
 }
+
+#[test_case(false ; "local backend")]
+#[test_case(true ; "git backend")]
+fn test_rename_remote(use_git: bool) {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init(use_git);
+    let repo = &test_repo.repo;
+    let mut tx = repo.start_transaction(&settings, "test");
+    let mut_repo = tx.mut_repo();
+    let commit = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let target = RefTarget::Normal(commit.id().clone());
+    mut_repo.set_remote_branch("main".to_string(), "origin".to_string(), target.clone());
+    mut_repo.rename_remote("origin", "upstream");
+    assert_eq!(mut_repo.get_remote_branch("main", "upstream"), Some(target));
+    assert_eq!(mut_repo.get_remote_branch("main", "origin"), None);
+}
