@@ -1,4 +1,4 @@
-// Copyright 2021 Google LLC
+// Copyright 2021 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,16 +12,22 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::path::Path;
+
 use jujutsu_lib::backend::{CommitId, MillisSinceEpoch, Signature, Timestamp};
 use jujutsu_lib::commit_builder::CommitBuilder;
-use jujutsu_lib::matchers::FilesMatcher;
+use jujutsu_lib::git;
+use jujutsu_lib::matchers::{FilesMatcher, Matcher};
 use jujutsu_lib::op_store::{RefTarget, WorkspaceId};
 use jujutsu_lib::repo::RepoRef;
 use jujutsu_lib::repo_path::RepoPath;
-use jujutsu_lib::revset::{self, parse, resolve_symbol, RevsetError, RevsetExpression};
-use jujutsu_lib::testutils::{CommitGraphBuilder, TestRepo};
-use jujutsu_lib::{git, testutils};
+use jujutsu_lib::revset::{
+    self, optimize, parse, resolve_symbol, RevsetAliasesMap, RevsetError, RevsetExpression,
+    RevsetWorkspaceContext,
+};
+use jujutsu_lib::workspace::Workspace;
 use test_case::test_case;
+use testutils::{create_random_commit, CommitGraphBuilder, TestRepo, TestWorkspace};
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -42,7 +48,7 @@ fn test_resolve_symbol_commit_id() {
     let test_repo = TestRepo::init(true);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let signature = Signature {
         name: "test".to_string(),
@@ -97,6 +103,12 @@ fn test_resolve_symbol_commit_id() {
         Ok(vec![commits[2].id().clone()])
     );
 
+    // Test empty commit id
+    assert_eq!(
+        resolve_symbol(repo_ref, "", None),
+        Err(RevsetError::AmbiguousIdPrefix("".to_string()))
+    );
+
     // Test commit id prefix
     assert_eq!(
         resolve_symbol(repo_ref, "046", None),
@@ -104,11 +116,11 @@ fn test_resolve_symbol_commit_id() {
     );
     assert_eq!(
         resolve_symbol(repo_ref, "04", None),
-        Err(RevsetError::AmbiguousCommitIdPrefix("04".to_string()))
+        Err(RevsetError::AmbiguousIdPrefix("04".to_string()))
     );
     assert_eq!(
         resolve_symbol(repo_ref, "", None),
-        Err(RevsetError::AmbiguousCommitIdPrefix("".to_string()))
+        Err(RevsetError::AmbiguousIdPrefix("".to_string()))
     );
     assert_eq!(
         resolve_symbol(repo_ref, "040", None),
@@ -120,10 +132,24 @@ fn test_resolve_symbol_commit_id() {
         resolve_symbol(repo_ref, "foo", None),
         Err(RevsetError::NoSuchRevision("foo".to_string()))
     );
+
+    // Test present() suppresses only NoSuchRevision error
+    assert_eq!(resolve_commit_ids(repo_ref, "present(foo)"), []);
+    assert_eq!(
+        optimize(parse("present(04)", &RevsetAliasesMap::new(), None).unwrap())
+            .evaluate(repo_ref, None)
+            .map(|_| ()),
+        Err(RevsetError::AmbiguousIdPrefix("04".to_string()))
+    );
+    assert_eq!(
+        resolve_commit_ids(repo_ref, "present(046)"),
+        vec![commits[2].id().clone()]
+    );
 }
 
 #[test]
 fn test_resolve_symbol_change_id() {
+    let settings = testutils::user_settings();
     // Test only with git so we can get predictable change ids
     let test_repo = TestRepo::init(true);
     let repo = &test_repo.repo;
@@ -145,7 +171,7 @@ fn test_resolve_symbol_change_id() {
     .unwrap();
     let git_tree = git_repo.find_tree(empty_tree_id).unwrap();
     let mut git_commit_ids = vec![];
-    for i in &[133, 664, 840] {
+    for i in &[133, 664, 840, 5085] {
         let git_commit_id = git_repo
             .commit(
                 Some(&format!("refs/heads/branch{}", i)),
@@ -159,7 +185,7 @@ fn test_resolve_symbol_change_id() {
         git_commit_ids.push(git_commit_id);
     }
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     git::import_refs(tx.mut_repo(), &git_repo).unwrap();
     let repo = tx.commit();
 
@@ -179,6 +205,11 @@ fn test_resolve_symbol_change_id() {
         // "04e1c7082e4e34f3f371d8a1a46770b861b9b547" reversed
         "e2ad9d861d0ee625851b8ecfcf2c727410e38720"
     );
+    assert_eq!(
+        hex::encode(git_commit_ids[3]),
+        // "911d7e52fd5ba04b8f289e14c3d30b52d38c0020" reversed
+        "040031cb4ad0cbc3287914f1d205dabf4a7eb889"
+    );
 
     // Test lookup by full change id
     let repo_ref = repo.as_repo_ref();
@@ -216,18 +247,29 @@ fn test_resolve_symbol_change_id() {
     );
     assert_eq!(
         resolve_symbol(repo_ref, "04e1", None),
-        Err(RevsetError::AmbiguousChangeIdPrefix("04e1".to_string()))
+        Err(RevsetError::AmbiguousIdPrefix("04e1".to_string()))
     );
     assert_eq!(
         resolve_symbol(repo_ref, "", None),
-        // Commit id is checked first, so this is considered an ambiguous commit id
-        Err(RevsetError::AmbiguousCommitIdPrefix("".to_string()))
+        Err(RevsetError::AmbiguousIdPrefix("".to_string()))
     );
     assert_eq!(
         resolve_symbol(repo_ref, "04e13", None),
         Err(RevsetError::NoSuchRevision("04e13".to_string()))
     );
 
+    // Test commit/changed id conflicts.
+    assert_eq!(
+        resolve_symbol(repo_ref, "040b", None),
+        Ok(vec![CommitId::from_hex(
+            "5339432b8e7b90bd3aa1a323db71b8a5c5dcd020"
+        )])
+    );
+    assert_eq!(
+        resolve_symbol(repo_ref, "040", None),
+        Err(RevsetError::AmbiguousIdPrefix("040".to_string()))
+    );
+
     // Test non-hex string
     assert_eq!(
         resolve_symbol(repo_ref, "foo", None),
@@ -242,11 +284,11 @@ fn test_resolve_symbol_checkout(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
 
     let ws1 = WorkspaceId::new("ws1".to_string());
     let ws2 = WorkspaceId::new("ws2".to_string());
@@ -266,8 +308,10 @@ fn test_resolve_symbol_checkout(use_git: bool) {
     );
 
     // Add some workspaces
-    mut_repo.set_wc_commit(ws1.clone(), commit1.id().clone());
-    mut_repo.set_wc_commit(ws2, commit2.id().clone());
+    mut_repo
+        .set_wc_commit(ws1.clone(), commit1.id().clone())
+        .unwrap();
+    mut_repo.set_wc_commit(ws2, commit2.id().clone()).unwrap();
     // @ cannot be resolved without a default workspace ID
     assert_eq!(
         resolve_symbol(mut_repo.as_repo_ref(), "@", None),
@@ -291,15 +335,15 @@ fn test_resolve_symbol_git_refs() {
     let test_repo = TestRepo::init(true);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     // Create some commits and refs to work with and so the repo is not empty
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit4 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit5 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit3 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit4 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit5 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_git_ref(
         "refs/heads/branch1".to_string(),
         RefTarget::Normal(commit1.id().clone()),
@@ -390,7 +434,9 @@ fn test_resolve_symbol_git_refs() {
 
     // Cannot shadow checkout ("@") or root symbols
     let ws_id = WorkspaceId::default();
-    mut_repo.set_wc_commit(ws_id.clone(), commit1.id().clone());
+    mut_repo
+        .set_wc_commit(ws_id.clone(), commit1.id().clone())
+        .unwrap();
     mut_repo.set_git_ref("@".to_string(), RefTarget::Normal(commit2.id().clone()));
     mut_repo.set_git_ref("root".to_string(), RefTarget::Normal(commit3.id().clone()));
     assert_eq!(
@@ -414,7 +460,7 @@ fn test_resolve_symbol_git_refs() {
 }
 
 fn resolve_commit_ids(repo: RepoRef, revset_str: &str) -> Vec<CommitId> {
-    let expression = parse(revset_str).unwrap();
+    let expression = optimize(parse(revset_str, &RevsetAliasesMap::new(), None).unwrap());
     expression
         .evaluate(repo, None)
         .unwrap()
@@ -426,11 +472,18 @@ fn resolve_commit_ids(repo: RepoRef, revset_str: &str) -> Vec<CommitId> {
 fn resolve_commit_ids_in_workspace(
     repo: RepoRef,
     revset_str: &str,
-    workspace_id: &WorkspaceId,
+    workspace: &Workspace,
+    cwd: Option<&Path>,
 ) -> Vec<CommitId> {
-    let expression = parse(revset_str).unwrap();
+    let workspace_ctx = RevsetWorkspaceContext {
+        cwd: cwd.unwrap_or_else(|| workspace.workspace_root()),
+        workspace_id: workspace.workspace_id(),
+        workspace_root: workspace.workspace_root(),
+    };
+    let expression =
+        optimize(parse(revset_str, &RevsetAliasesMap::new(), Some(&workspace_ctx)).unwrap());
     expression
-        .evaluate(repo, Some(workspace_id))
+        .evaluate(repo, Some(&workspace_ctx))
         .unwrap()
         .iter()
         .commit_ids()
@@ -441,14 +494,14 @@ fn resolve_commit_ids_in_workspace(
 #[test_case(true ; "git backend")]
 fn test_evaluate_expression_root_and_checkout(use_git: bool) {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
-    let repo = &test_repo.repo;
+    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let repo = &test_workspace.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     let root_commit = repo.store().root_commit();
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
 
     // Can find the root commit
     assert_eq!(
@@ -457,9 +510,16 @@ fn test_evaluate_expression_root_and_checkout(use_git: bool) {
     );
 
     // Can find the current checkout
-    mut_repo.set_wc_commit(WorkspaceId::default(), commit1.id().clone());
+    mut_repo
+        .set_wc_commit(WorkspaceId::default(), commit1.id().clone())
+        .unwrap();
     assert_eq!(
-        resolve_commit_ids_in_workspace(mut_repo.as_repo_ref(), "@", &WorkspaceId::default()),
+        resolve_commit_ids_in_workspace(
+            mut_repo.as_repo_ref(),
+            "@",
+            &test_workspace.workspace,
+            None,
+        ),
         vec![commit1.id().clone()]
     );
 }
@@ -472,7 +532,7 @@ fn test_evaluate_expression_heads(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -534,7 +594,7 @@ fn test_evaluate_expression_roots(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -592,11 +652,11 @@ fn test_evaluate_expression_roots(use_git: bool) {
 #[test_case(true ; "git backend")]
 fn test_evaluate_expression_parents(use_git: bool) {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
-    let repo = &test_repo.repo;
+    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let repo = &test_workspace.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -609,9 +669,16 @@ fn test_evaluate_expression_parents(use_git: bool) {
     assert_eq!(resolve_commit_ids(mut_repo.as_repo_ref(), "root-"), vec![]);
 
     // Can find parents of the current checkout
-    mut_repo.set_wc_commit(WorkspaceId::default(), commit2.id().clone());
+    mut_repo
+        .set_wc_commit(WorkspaceId::default(), commit2.id().clone())
+        .unwrap();
     assert_eq!(
-        resolve_commit_ids_in_workspace(mut_repo.as_repo_ref(), "@-", &WorkspaceId::default()),
+        resolve_commit_ids_in_workspace(
+            mut_repo.as_repo_ref(),
+            "@-",
+            &test_workspace.workspace,
+            None,
+        ),
         vec![commit1.id().clone()]
     );
 
@@ -656,20 +723,20 @@ fn test_evaluate_expression_children(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo)
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo)
+    let commit3 = create_random_commit(&settings, repo)
         .set_parents(vec![commit2.id().clone()])
         .write_to_repo(mut_repo);
-    let commit4 = testutils::create_random_commit(&settings, repo)
+    let commit4 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .write_to_repo(mut_repo);
-    let commit5 = testutils::create_random_commit(&settings, repo)
+    let commit5 = create_random_commit(&settings, repo)
         .set_parents(vec![commit3.id().clone(), commit4.id().clone()])
         .write_to_repo(mut_repo);
 
@@ -711,7 +778,7 @@ fn test_evaluate_expression_ancestors(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -746,7 +813,7 @@ fn test_evaluate_expression_range(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -810,7 +877,7 @@ fn test_evaluate_expression_dag_range(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit_id = repo.store().root_commit_id().clone();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -880,7 +947,7 @@ fn test_evaluate_expression_connected(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit_id = repo.store().root_commit_id().clone();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -959,21 +1026,21 @@ fn test_evaluate_expression_descendants(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     let root_commit_id = repo.store().root_commit_id().clone();
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo)
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo)
+    let commit3 = create_random_commit(&settings, repo)
         .set_parents(vec![commit2.id().clone()])
         .write_to_repo(mut_repo);
-    let commit4 = testutils::create_random_commit(&settings, repo)
+    let commit4 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .write_to_repo(mut_repo);
-    let commit5 = testutils::create_random_commit(&settings, repo)
+    let commit5 = create_random_commit(&settings, repo)
         .set_parents(vec![commit3.id().clone(), commit4.id().clone()])
         .write_to_repo(mut_repo);
 
@@ -1018,7 +1085,7 @@ fn test_evaluate_expression_all(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let root_commit_id = repo.store().root_commit_id().clone();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
@@ -1046,7 +1113,7 @@ fn test_evaluate_expression_visible_heads(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -1067,7 +1134,7 @@ fn test_evaluate_expression_public_heads(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -1099,13 +1166,13 @@ fn test_evaluate_expression_git_refs(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit4 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit3 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit4 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
 
     // Can get git refs when there are none
     assert_eq!(
@@ -1168,10 +1235,10 @@ fn test_evaluate_expression_git_head(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
 
     // Can get git head when it's not set
     assert_eq!(
@@ -1192,13 +1259,13 @@ fn test_evaluate_expression_branches(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit4 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit3 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit4 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
 
     // Can get branches when there are none
     assert_eq!(
@@ -1261,13 +1328,13 @@ fn test_evaluate_expression_remote_branches(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let commit4 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit3 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let commit4 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
 
     // Can get branches when there are none
     assert_eq!(
@@ -1341,7 +1408,7 @@ fn test_evaluate_expression_merges(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -1359,7 +1426,7 @@ fn test_evaluate_expression_merges(use_git: bool) {
     assert_eq!(
         resolve_commit_ids(
             mut_repo.as_repo_ref(),
-            &format!("merges(:{})", commit5.id().hex())
+            &format!(":{} & merges()", commit5.id().hex())
         ),
         vec![commit5.id().clone()]
     );
@@ -1372,17 +1439,17 @@ fn test_evaluate_expression_description(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
-    let commit1 = testutils::create_random_commit(&settings, repo)
+    let commit1 = create_random_commit(&settings, repo)
         .set_description("commit 1".to_string())
         .write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo)
+    let commit2 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .set_description("commit 2".to_string())
         .write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo)
+    let commit3 = create_random_commit(&settings, repo)
         .set_parents(vec![commit2.id().clone()])
         .set_description("commit 3".to_string())
         .write_to_repo(mut_repo);
@@ -1403,7 +1470,10 @@ fn test_evaluate_expression_description(use_git: bool) {
     );
     // Searches only among candidates if specified
     assert_eq!(
-        resolve_commit_ids(mut_repo.as_repo_ref(), "description(\"commit 2\",heads())"),
+        resolve_commit_ids(
+            mut_repo.as_repo_ref(),
+            "heads() & description(\"commit 2\")"
+        ),
         vec![]
     );
 }
@@ -1415,21 +1485,21 @@ fn test_evaluate_expression_author(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     let timestamp = Timestamp {
         timestamp: MillisSinceEpoch(0),
         tz_offset: 0,
     };
-    let commit1 = testutils::create_random_commit(&settings, repo)
+    let commit1 = create_random_commit(&settings, repo)
         .set_author(Signature {
             name: "name1".to_string(),
             email: "email1".to_string(),
             timestamp: timestamp.clone(),
         })
         .write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo)
+    let commit2 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .set_author(Signature {
             name: "name2".to_string(),
@@ -1437,7 +1507,7 @@ fn test_evaluate_expression_author(use_git: bool) {
             timestamp: timestamp.clone(),
         })
         .write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo)
+    let commit3 = create_random_commit(&settings, repo)
         .set_parents(vec![commit2.id().clone()])
         .set_author(Signature {
             name: "name3".to_string(),
@@ -1466,7 +1536,7 @@ fn test_evaluate_expression_author(use_git: bool) {
     );
     // Searches only among candidates if specified
     assert_eq!(
-        resolve_commit_ids(mut_repo.as_repo_ref(), "author(\"name2\",heads())"),
+        resolve_commit_ids(mut_repo.as_repo_ref(), "heads() & author(\"name2\")"),
         vec![]
     );
 }
@@ -1478,21 +1548,21 @@ fn test_evaluate_expression_committer(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     let timestamp = Timestamp {
         timestamp: MillisSinceEpoch(0),
         tz_offset: 0,
     };
-    let commit1 = testutils::create_random_commit(&settings, repo)
+    let commit1 = create_random_commit(&settings, repo)
         .set_committer(Signature {
             name: "name1".to_string(),
             email: "email1".to_string(),
             timestamp: timestamp.clone(),
         })
         .write_to_repo(mut_repo);
-    let commit2 = testutils::create_random_commit(&settings, repo)
+    let commit2 = create_random_commit(&settings, repo)
         .set_parents(vec![commit1.id().clone()])
         .set_committer(Signature {
             name: "name2".to_string(),
@@ -1500,7 +1570,7 @@ fn test_evaluate_expression_committer(use_git: bool) {
             timestamp: timestamp.clone(),
         })
         .write_to_repo(mut_repo);
-    let commit3 = testutils::create_random_commit(&settings, repo)
+    let commit3 = create_random_commit(&settings, repo)
         .set_parents(vec![commit2.id().clone()])
         .set_committer(Signature {
             name: "name3".to_string(),
@@ -1529,7 +1599,7 @@ fn test_evaluate_expression_committer(use_git: bool) {
     );
     // Searches only among candidates if specified
     assert_eq!(
-        resolve_commit_ids(mut_repo.as_repo_ref(), "committer(\"name2\",heads())"),
+        resolve_commit_ids(mut_repo.as_repo_ref(), "heads() & committer(\"name2\")"),
         vec![]
     );
 }
@@ -1542,7 +1612,7 @@ fn test_evaluate_expression_union(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -1615,7 +1685,7 @@ fn test_evaluate_expression_intersection(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -1655,7 +1725,7 @@ fn test_evaluate_expression_difference(use_git: bool) {
     let repo = &test_repo.repo;
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit1 = graph_builder.initial_commit();
@@ -1664,6 +1734,12 @@ fn test_evaluate_expression_difference(use_git: bool) {
     let commit4 = graph_builder.commit_with_parents(&[&commit3]);
     let commit5 = graph_builder.commit_with_parents(&[&commit2]);
 
+    // Difference from all
+    assert_eq!(
+        resolve_commit_ids(mut_repo.as_repo_ref(), &format!("~:{}", commit5.id().hex())),
+        vec![commit4.id().clone(), commit3.id().clone()]
+    );
+
     // Difference between ancestors
     assert_eq!(
         resolve_commit_ids(
@@ -1679,6 +1755,13 @@ fn test_evaluate_expression_difference(use_git: bool) {
         ),
         vec![commit5.id().clone()]
     );
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo.as_repo_ref(),
+            &format!("~:{} & :{}", commit4.id().hex(), commit5.id().hex())
+        ),
+        vec![commit5.id().clone()]
+    );
     assert_eq!(
         resolve_commit_ids(
             mut_repo.as_repo_ref(),
@@ -1725,10 +1808,10 @@ fn test_evaluate_expression_difference(use_git: bool) {
 #[test_case(true ; "git backend")]
 fn test_filter_by_diff(use_git: bool) {
     let settings = testutils::user_settings();
-    let test_repo = TestRepo::init(use_git);
-    let repo = &test_repo.repo;
+    let test_workspace = TestWorkspace::init(&settings, use_git);
+    let repo = &test_workspace.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
 
     let added_clean_clean = RepoPath::from_internal_string("added_clean_clean");
@@ -1770,12 +1853,16 @@ fn test_filter_by_diff(use_git: bool) {
     let commit3 =
         CommitBuilder::for_new_commit(&settings, vec![commit2.id().clone()], tree3.id().clone())
             .write_to_repo(mut_repo);
+    let commit4 =
+        CommitBuilder::for_new_commit(&settings, vec![commit3.id().clone()], tree3.id().clone())
+            .write_to_repo(mut_repo);
 
+    // matcher API:
     let resolve = |file_path: &RepoPath| -> Vec<CommitId> {
         let repo_ref = mut_repo.as_repo_ref();
         let matcher = FilesMatcher::new([file_path.clone()].into());
         let candidates = RevsetExpression::all().evaluate(repo_ref, None).unwrap();
-        let commit_ids = revset::filter_by_diff(repo_ref, &matcher, candidates)
+        let commit_ids = revset::filter_by_diff(repo_ref, &matcher as &dyn Matcher, candidates)
             .iter()
             .commit_ids()
             .collect();
@@ -1795,4 +1882,33 @@ fn test_filter_by_diff(use_git: bool) {
             commit1.id().clone()
         ]
     );
+
+    // file() revset:
+    assert_eq!(
+        resolve_commit_ids_in_workspace(
+            mut_repo.as_repo_ref(),
+            r#"file("repo/added_clean_clean")"#,
+            &test_workspace.workspace,
+            Some(test_workspace.workspace.workspace_root().parent().unwrap()),
+        ),
+        vec![commit1.id().clone()]
+    );
+    assert_eq!(
+        resolve_commit_ids_in_workspace(
+            mut_repo.as_repo_ref(),
+            &format!(r#"{}: & file("added_modified_clean")"#, commit2.id().hex()),
+            &test_workspace.workspace,
+            Some(test_workspace.workspace.workspace_root()),
+        ),
+        vec![commit2.id().clone()]
+    );
+
+    // empty() revset, which is identical to ~file(".")
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo.as_repo_ref(),
+            &format!("{}: & empty()", commit1.id().hex())
+        ),
+        vec![commit4.id().clone()]
+    );
 }
