@@ -1,4 +1,4 @@
-// Copyright 2021 Google LLC
+// Copyright 2021 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -16,10 +16,9 @@ use jujutsu_lib::commit_builder::CommitBuilder;
 use jujutsu_lib::op_store::{RefTarget, WorkspaceId};
 use jujutsu_lib::repo_path::RepoPath;
 use jujutsu_lib::rewrite::DescendantRebaser;
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::{assert_rebased, create_random_commit, CommitGraphBuilder, TestRepo};
 use maplit::{hashmap, hashset};
 use test_case::test_case;
+use testutils::{assert_rebased, create_random_commit, CommitGraphBuilder, TestRepo};
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -37,7 +36,7 @@ fn test_rebase_descendants_sideways(use_git: bool) {
     // | B
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -93,7 +92,7 @@ fn test_rebase_descendants_forward(use_git: bool) {
     // |/
     // B
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -147,7 +146,7 @@ fn test_rebase_descendants_reorder(use_git: bool) {
     // |/
     // B
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -194,7 +193,7 @@ fn test_rebase_descendants_backward(use_git: bool) {
     // C
     // B
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -236,7 +235,7 @@ fn test_rebase_descendants_chain_becomes_branchy(use_git: bool) {
     // B E
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -284,7 +283,7 @@ fn test_rebase_descendants_internal_merge(use_git: bool) {
     // | B
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -335,7 +334,7 @@ fn test_rebase_descendants_external_merge(use_git: bool) {
     // | B
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -382,7 +381,7 @@ fn test_rebase_descendants_abandon(use_git: bool) {
     // |/
     // B
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -424,7 +423,7 @@ fn test_rebase_descendants_abandon_no_descendants(use_git: bool) {
     // C
     // B
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -462,7 +461,7 @@ fn test_rebase_descendants_abandon_and_replace(use_git: bool) {
     // E B
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -501,7 +500,7 @@ fn test_rebase_descendants_abandon_degenerate_merge(use_git: bool) {
     // B C
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -541,7 +540,7 @@ fn test_rebase_descendants_abandon_widen_merge(use_git: bool) {
     // B C D
     //  \|/
     //   A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -585,7 +584,7 @@ fn test_rebase_descendants_multiple_sideways(use_git: bool) {
     // | |/
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -631,7 +630,7 @@ fn test_rebase_descendants_multiple_swap(use_git: bool) {
     // B D
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -674,7 +673,7 @@ fn test_rebase_descendants_multiple_no_descendants(use_git: bool) {
     // B C
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -728,7 +727,7 @@ fn test_rebase_descendants_divergent_rewrite(use_git: bool) {
     // | B2
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -787,7 +786,7 @@ fn test_rebase_descendants_repeated(use_git: bool) {
     // | B2
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -846,7 +845,7 @@ fn test_rebase_descendants_contents(use_git: bool) {
     // | B
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let path1 = RepoPath::from_internal_string("file1");
     let tree1 = testutils::create_tree(repo, &[(&path1, "content")]);
     let commit_a = CommitBuilder::for_new_commit(
@@ -913,7 +912,7 @@ fn test_rebase_descendants_basic_branch_update() {
     // B main         B2 main
     // |         =>   |
     // A              A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -921,7 +920,7 @@ fn test_rebase_descendants_basic_branch_update() {
         .set_local_branch("main".to_string(), RefTarget::Normal(commit_b.id().clone()));
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_b2 =
         CommitBuilder::for_rewrite_from(&settings, &commit_b).write_to_repo(tx.mut_repo());
     tx.mut_repo().rebase_descendants(&settings).unwrap();
@@ -952,7 +951,7 @@ fn test_rebase_descendants_branch_move_two_steps() {
     // B B2           B2
     // |/             |
     // A              A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -961,7 +960,7 @@ fn test_rebase_descendants_branch_move_two_steps() {
         .set_local_branch("main".to_string(), RefTarget::Normal(commit_c.id().clone()));
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_b2 =
         CommitBuilder::for_rewrite_from(&settings, &commit_b).write_to_repo(tx.mut_repo());
     let commit_c2 =
@@ -993,7 +992,7 @@ fn test_rebase_descendants_basic_branch_update_with_non_local_branch() {
     // B main main@origin v1          | B main@origin v1
     // |                         =>   |/
     // A                              A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -1008,7 +1007,7 @@ fn test_rebase_descendants_basic_branch_update_with_non_local_branch() {
         .set_tag("v1".to_string(), RefTarget::Normal(commit_b.id().clone()));
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_b2 =
         CommitBuilder::for_rewrite_from(&settings, &commit_b).write_to_repo(tx.mut_repo());
     tx.mut_repo().rebase_descendants(&settings).unwrap();
@@ -1046,7 +1045,7 @@ fn test_rebase_descendants_update_branch_after_abandon() {
     // B main
     // |          =>   A main
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -1054,7 +1053,7 @@ fn test_rebase_descendants_update_branch_after_abandon() {
         .set_local_branch("main".to_string(), RefTarget::Normal(commit_b.id().clone()));
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     tx.mut_repo().record_abandoned_commit(commit_b.id().clone());
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     assert_eq!(
@@ -1082,7 +1081,7 @@ fn test_rebase_descendants_update_branches_after_divergent_rewrite() {
     // B main         |/B2 main?
     // |         =>   |/
     // A              A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -1090,7 +1089,7 @@ fn test_rebase_descendants_update_branches_after_divergent_rewrite() {
         .set_local_branch("main".to_string(), RefTarget::Normal(commit_b.id().clone()));
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_b2 =
         CommitBuilder::for_rewrite_from(&settings, &commit_b).write_to_repo(tx.mut_repo());
     // Different description so they're not the same commit
@@ -1133,7 +1132,7 @@ fn test_rebase_descendants_rewrite_updates_branch_conflict() {
     // Branch "main" is a conflict removing commit A and adding commits B and C.
     // A gets rewritten as A2 and A3. B gets rewritten as B2 and B2. The branch
     // should become a conflict removing A and B, and adding B2, B3, C.
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.initial_commit();
@@ -1147,7 +1146,7 @@ fn test_rebase_descendants_rewrite_updates_branch_conflict() {
     );
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_a2 =
         CommitBuilder::for_rewrite_from(&settings, &commit_a).write_to_repo(tx.mut_repo());
     // Different description so they're not the same commit
@@ -1198,7 +1197,7 @@ fn test_rebase_descendants_rewrite_resolves_branch_conflict() {
     // would result in a conflict removing A and adding B2 and C. However, since C
     // is a descendant of A, and B2 is a descendant of C, the conflict gets
     // resolved to B2.
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -1212,7 +1211,7 @@ fn test_rebase_descendants_rewrite_resolves_branch_conflict() {
     );
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_b2 = CommitBuilder::for_rewrite_from(&settings, &commit_b)
         .set_parents(vec![commit_c.id().clone()])
         .write_to_repo(tx.mut_repo());
@@ -1239,7 +1238,7 @@ fn test_rebase_descendants_branch_delete_modify_abandon() {
     // leaves the branch pointing to "-A+B". We now abandon B. That should
     // result in the branch pointing to "-A+A=0", so the branch should
     // be deleted.
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -1252,7 +1251,7 @@ fn test_rebase_descendants_branch_delete_modify_abandon() {
     );
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     tx.mut_repo().record_abandoned_commit(commit_b.id().clone());
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     assert_eq!(tx.mut_repo().get_local_branch("main"), None);
@@ -1271,24 +1270,26 @@ fn test_rebase_descendants_update_checkout(use_git: bool) {
     // C B
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_a = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let commit_b = create_random_commit(&settings, repo)
         .set_parents(vec![commit_a.id().clone()])
-        .set_open(true)
         .write_to_repo(tx.mut_repo());
     let ws1_id = WorkspaceId::new("ws1".to_string());
     let ws2_id = WorkspaceId::new("ws2".to_string());
     let ws3_id = WorkspaceId::new("ws3".to_string());
     tx.mut_repo()
-        .set_wc_commit(ws1_id.clone(), commit_b.id().clone());
+        .set_wc_commit(ws1_id.clone(), commit_b.id().clone())
+        .unwrap();
     tx.mut_repo()
-        .set_wc_commit(ws2_id.clone(), commit_b.id().clone());
+        .set_wc_commit(ws2_id.clone(), commit_b.id().clone())
+        .unwrap();
     tx.mut_repo()
-        .set_wc_commit(ws3_id.clone(), commit_a.id().clone());
+        .set_wc_commit(ws3_id.clone(), commit_a.id().clone())
+        .unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_c = CommitBuilder::for_rewrite_from(&settings, &commit_b)
         .set_description("C".to_string())
         .write_to_repo(tx.mut_repo());
@@ -1309,30 +1310,32 @@ fn test_rebase_descendants_update_checkout_abandoned(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    // Checked-out, open commit B was abandoned. A child of A
+    // Checked-out commit B was abandoned. A child of A
     // should become checked out.
     //
     // B
     // |
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_a = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let commit_b = create_random_commit(&settings, repo)
         .set_parents(vec![commit_a.id().clone()])
-        .set_open(true)
         .write_to_repo(tx.mut_repo());
     let ws1_id = WorkspaceId::new("ws1".to_string());
     let ws2_id = WorkspaceId::new("ws2".to_string());
     let ws3_id = WorkspaceId::new("ws3".to_string());
     tx.mut_repo()
-        .set_wc_commit(ws1_id.clone(), commit_b.id().clone());
+        .set_wc_commit(ws1_id.clone(), commit_b.id().clone())
+        .unwrap();
     tx.mut_repo()
-        .set_wc_commit(ws2_id.clone(), commit_b.id().clone());
+        .set_wc_commit(ws2_id.clone(), commit_b.id().clone())
+        .unwrap();
     tx.mut_repo()
-        .set_wc_commit(ws3_id.clone(), commit_a.id().clone());
+        .set_wc_commit(ws3_id.clone(), commit_a.id().clone())
+        .unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     tx.mut_repo().record_abandoned_commit(commit_b.id().clone());
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     let repo = tx.commit();
@@ -1347,7 +1350,6 @@ fn test_rebase_descendants_update_checkout_abandoned(use_git: bool) {
         .store()
         .get_commit(repo.view().get_wc_commit_id(&ws1_id).unwrap())
         .unwrap();
-    assert!(checkout.is_open());
     assert_eq!(checkout.parent_ids(), vec![commit_a.id().clone()]);
     assert_eq!(repo.view().get_wc_commit_id(&ws3_id), Some(commit_a.id()));
 }
@@ -1359,7 +1361,7 @@ fn test_rebase_descendants_update_checkout_abandoned_merge(use_git: bool) {
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
 
-    // Checked-out, open merge commit D was abandoned. A parent commit should become
+    // Checked-out merge commit D was abandoned. A parent commit should become
     // checked out.
     //
     // D
@@ -1367,7 +1369,7 @@ fn test_rebase_descendants_update_checkout_abandoned_merge(use_git: bool) {
     // B C
     // |/
     // A
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let commit_a = create_random_commit(&settings, repo).write_to_repo(tx.mut_repo());
     let commit_b = create_random_commit(&settings, repo)
         .set_parents(vec![commit_a.id().clone()])
@@ -1377,20 +1379,19 @@ fn test_rebase_descendants_update_checkout_abandoned_merge(use_git: bool) {
         .write_to_repo(tx.mut_repo());
     let commit_d = create_random_commit(&settings, repo)
         .set_parents(vec![commit_b.id().clone(), commit_c.id().clone()])
-        .set_open(true)
         .write_to_repo(tx.mut_repo());
     let workspace_id = WorkspaceId::default();
     tx.mut_repo()
-        .set_wc_commit(workspace_id.clone(), commit_d.id().clone());
+        .set_wc_commit(workspace_id.clone(), commit_d.id().clone())
+        .unwrap();
     let repo = tx.commit();
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     tx.mut_repo().record_abandoned_commit(commit_d.id().clone());
     tx.mut_repo().rebase_descendants(&settings).unwrap();
     let repo = tx.commit();
 
     let new_checkout_id = repo.view().get_wc_commit_id(&workspace_id).unwrap();
     let checkout = repo.store().get_commit(new_checkout_id).unwrap();
-    assert!(checkout.is_open());
     assert_eq!(checkout.parent_ids(), vec![commit_b.id().clone()]);
 }
