@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -18,11 +18,10 @@ use jujutsu_lib::commit_builder::CommitBuilder;
 use jujutsu_lib::op_store::{BranchTarget, RefTarget, WorkspaceId};
 use jujutsu_lib::repo::ReadonlyRepo;
 use jujutsu_lib::settings::UserSettings;
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::{CommitGraphBuilder, TestRepo};
 use jujutsu_lib::transaction::Transaction;
 use maplit::{btreemap, hashset};
 use test_case::test_case;
+use testutils::{create_random_commit, CommitGraphBuilder, TestRepo};
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -46,7 +45,7 @@ fn test_heads_fork(use_git: bool) {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
 
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let initial = graph_builder.initial_commit();
@@ -69,7 +68,7 @@ fn test_heads_merge(use_git: bool) {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(use_git);
     let repo = &test_repo.repo;
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
 
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let initial = graph_builder.initial_commit();
@@ -88,41 +87,34 @@ fn test_merge_views_heads() {
     let test_repo = TestRepo::init(false);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let head_unchanged = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let head_remove_tx1 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let head_remove_tx2 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let public_head_unchanged =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let head_unchanged = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let head_remove_tx1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let head_remove_tx2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let public_head_unchanged = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.add_public_head(&public_head_unchanged);
-    let public_head_remove_tx1 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let public_head_remove_tx1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.add_public_head(&public_head_remove_tx1);
-    let public_head_remove_tx2 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let public_head_remove_tx2 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.add_public_head(&public_head_remove_tx2);
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
+    let mut tx1 = repo.start_transaction(&settings, "test");
     tx1.mut_repo().remove_head(head_remove_tx1.id());
     tx1.mut_repo()
         .remove_public_head(public_head_remove_tx1.id());
-    let head_add_tx1 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
-    let public_head_add_tx1 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let head_add_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let public_head_add_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo().add_public_head(&public_head_add_tx1);
     tx1.commit();
 
-    let mut tx2 = repo.start_transaction("test");
+    let mut tx2 = repo.start_transaction(&settings, "test");
     tx2.mut_repo().remove_head(head_remove_tx2.id());
     tx2.mut_repo()
         .remove_public_head(public_head_remove_tx2.id());
-    let head_add_tx2 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
-    let public_head_add_tx2 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
+    let head_add_tx2 = create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
+    let public_head_add_tx2 = create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
     tx2.mut_repo().add_public_head(&public_head_add_tx2);
     tx2.commit();
 
@@ -162,13 +154,10 @@ fn test_merge_views_checkout() {
     // Workspace 5 gets deleted in tx2 and modified in tx1.
     // Workspace 6 gets added in tx1.
     // Workspace 7 gets added in tx2.
-    let mut initial_tx = repo.start_transaction("test");
-    let commit1 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(initial_tx.mut_repo());
-    let commit2 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(initial_tx.mut_repo());
-    let commit3 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(initial_tx.mut_repo());
+    let mut initial_tx = repo.start_transaction(&settings, "test");
+    let commit1 = create_random_commit(&settings, repo).write_to_repo(initial_tx.mut_repo());
+    let commit2 = create_random_commit(&settings, repo).write_to_repo(initial_tx.mut_repo());
+    let commit3 = create_random_commit(&settings, repo).write_to_repo(initial_tx.mut_repo());
     let ws1_id = WorkspaceId::new("ws1".to_string());
     let ws2_id = WorkspaceId::new("ws2".to_string());
     let ws3_id = WorkspaceId::new("ws3".to_string());
@@ -178,43 +167,56 @@ fn test_merge_views_checkout() {
     let ws7_id = WorkspaceId::new("ws7".to_string());
     initial_tx
         .mut_repo()
-        .set_wc_commit(ws1_id.clone(), commit1.id().clone());
+        .set_wc_commit(ws1_id.clone(), commit1.id().clone())
+        .unwrap();
     initial_tx
         .mut_repo()
-        .set_wc_commit(ws2_id.clone(), commit1.id().clone());
+        .set_wc_commit(ws2_id.clone(), commit1.id().clone())
+        .unwrap();
     initial_tx
         .mut_repo()
-        .set_wc_commit(ws3_id.clone(), commit1.id().clone());
+        .set_wc_commit(ws3_id.clone(), commit1.id().clone())
+        .unwrap();
     initial_tx
         .mut_repo()
-        .set_wc_commit(ws4_id.clone(), commit1.id().clone());
+        .set_wc_commit(ws4_id.clone(), commit1.id().clone())
+        .unwrap();
     initial_tx
         .mut_repo()
-        .set_wc_commit(ws5_id.clone(), commit1.id().clone());
+        .set_wc_commit(ws5_id.clone(), commit1.id().clone())
+        .unwrap();
     let repo = initial_tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
+    let mut tx1 = repo.start_transaction(&settings, "test");
     tx1.mut_repo()
-        .set_wc_commit(ws1_id.clone(), commit2.id().clone());
+        .set_wc_commit(ws1_id.clone(), commit2.id().clone())
+        .unwrap();
     tx1.mut_repo()
-        .set_wc_commit(ws2_id.clone(), commit2.id().clone());
+        .set_wc_commit(ws2_id.clone(), commit2.id().clone())
+        .unwrap();
     tx1.mut_repo().remove_wc_commit(&ws4_id);
     tx1.mut_repo()
-        .set_wc_commit(ws5_id.clone(), commit2.id().clone());
+        .set_wc_commit(ws5_id.clone(), commit2.id().clone())
+        .unwrap();
     tx1.mut_repo()
-        .set_wc_commit(ws6_id.clone(), commit2.id().clone());
+        .set_wc_commit(ws6_id.clone(), commit2.id().clone())
+        .unwrap();
     tx1.commit();
 
-    let mut tx2 = repo.start_transaction("test");
+    let mut tx2 = repo.start_transaction(&settings, "test");
     tx2.mut_repo()
-        .set_wc_commit(ws1_id.clone(), commit3.id().clone());
+        .set_wc_commit(ws1_id.clone(), commit3.id().clone())
+        .unwrap();
     tx2.mut_repo()
-        .set_wc_commit(ws3_id.clone(), commit3.id().clone());
+        .set_wc_commit(ws3_id.clone(), commit3.id().clone())
+        .unwrap();
     tx2.mut_repo()
-        .set_wc_commit(ws4_id.clone(), commit3.id().clone());
+        .set_wc_commit(ws4_id.clone(), commit3.id().clone())
+        .unwrap();
     tx2.mut_repo().remove_wc_commit(&ws5_id);
     tx2.mut_repo()
-        .set_wc_commit(ws7_id.clone(), commit3.id().clone());
+        .set_wc_commit(ws7_id.clone(), commit3.id().clone())
+        .unwrap();
     // Make sure the end time different, assuming the clock has sub-millisecond
     // precision.
     std::thread::sleep(std::time::Duration::from_millis(1));
@@ -241,16 +243,12 @@ fn test_merge_views_branches() {
     let test_repo = TestRepo::init(false);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let main_branch_local_tx0 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let main_branch_origin_tx0 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let main_branch_origin_tx1 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
-    let main_branch_alternate_tx0 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let main_branch_local_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let main_branch_origin_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let main_branch_origin_tx1 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let main_branch_alternate_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_local_branch(
         "main".to_string(),
         RefTarget::Normal(main_branch_local_tx0.id().clone()),
@@ -265,17 +263,16 @@ fn test_merge_views_branches() {
         "alternate".to_string(),
         RefTarget::Normal(main_branch_alternate_tx0.id().clone()),
     );
-    let feature_branch_local_tx0 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let feature_branch_local_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_git_ref(
         "feature".to_string(),
         RefTarget::Normal(feature_branch_local_tx0.id().clone()),
     );
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
+    let mut tx1 = repo.start_transaction(&settings, "test");
     let main_branch_local_tx1 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+        create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo().set_local_branch(
         "main".to_string(),
         RefTarget::Normal(main_branch_local_tx1.id().clone()),
@@ -285,17 +282,16 @@ fn test_merge_views_branches() {
         "origin".to_string(),
         RefTarget::Normal(main_branch_origin_tx1.id().clone()),
     );
-    let feature_branch_tx1 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let feature_branch_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo().set_local_branch(
         "feature".to_string(),
         RefTarget::Normal(feature_branch_tx1.id().clone()),
     );
     tx1.commit();
 
-    let mut tx2 = repo.start_transaction("test");
+    let mut tx2 = repo.start_transaction(&settings, "test");
     let main_branch_local_tx2 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
+        create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
     tx2.mut_repo().set_local_branch(
         "main".to_string(),
         RefTarget::Normal(main_branch_local_tx2.id().clone()),
@@ -342,25 +338,25 @@ fn test_merge_views_tags() {
     let test_repo = TestRepo::init(false);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let v1_tx0 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let v1_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_tag("v1.0".to_string(), RefTarget::Normal(v1_tx0.id().clone()));
-    let v2_tx0 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let v2_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_tag("v2.0".to_string(), RefTarget::Normal(v2_tx0.id().clone()));
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
-    let v1_tx1 = testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let mut tx1 = repo.start_transaction(&settings, "test");
+    let v1_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo()
         .set_tag("v1.0".to_string(), RefTarget::Normal(v1_tx1.id().clone()));
-    let v2_tx1 = testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let v2_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo()
         .set_tag("v2.0".to_string(), RefTarget::Normal(v2_tx1.id().clone()));
     tx1.commit();
 
-    let mut tx2 = repo.start_transaction("test");
-    let v1_tx2 = testutils::create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
+    let mut tx2 = repo.start_transaction(&settings, "test");
+    let v1_tx2 = create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
     tx2.mut_repo()
         .set_tag("v1.0".to_string(), RefTarget::Normal(v1_tx2.id().clone()));
     tx2.commit();
@@ -388,39 +384,35 @@ fn test_merge_views_git_refs() {
     let test_repo = TestRepo::init(false);
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction("test");
+    let mut tx = repo.start_transaction(&settings, "test");
     let mut_repo = tx.mut_repo();
-    let main_branch_tx0 = testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let main_branch_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_git_ref(
         "refs/heads/main".to_string(),
         RefTarget::Normal(main_branch_tx0.id().clone()),
     );
-    let feature_branch_tx0 =
-        testutils::create_random_commit(&settings, repo).write_to_repo(mut_repo);
+    let feature_branch_tx0 = create_random_commit(&settings, repo).write_to_repo(mut_repo);
     mut_repo.set_git_ref(
         "refs/heads/feature".to_string(),
         RefTarget::Normal(feature_branch_tx0.id().clone()),
     );
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
-    let main_branch_tx1 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let mut tx1 = repo.start_transaction(&settings, "test");
+    let main_branch_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo().set_git_ref(
         "refs/heads/main".to_string(),
         RefTarget::Normal(main_branch_tx1.id().clone()),
     );
-    let feature_branch_tx1 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
+    let feature_branch_tx1 = create_random_commit(&settings, &repo).write_to_repo(tx1.mut_repo());
     tx1.mut_repo().set_git_ref(
         "refs/heads/feature".to_string(),
         RefTarget::Normal(feature_branch_tx1.id().clone()),
     );
     tx1.commit();
 
-    let mut tx2 = repo.start_transaction("test");
-    let main_branch_tx2 =
-        testutils::create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
+    let mut tx2 = repo.start_transaction(&settings, "test");
+    let main_branch_tx2 = create_random_commit(&settings, &repo).write_to_repo(tx2.mut_repo());
     tx2.mut_repo().set_git_ref(
         "refs/heads/main".to_string(),
         RefTarget::Normal(main_branch_tx2.id().clone()),
@@ -459,6 +451,38 @@ fn commit_transactions(settings: &UserSettings, txs: Vec<Transaction>) -> Arc<Re
     repo
 }
 
+#[test]
+fn test_merge_views_divergent() {
+    // We start with just commit A. Operation 1 rewrites it as A2. Operation 2
+    // rewrites it as A3.
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init(false);
+
+    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let commit_a = create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
+    let repo = tx.commit();
+
+    let mut tx1 = repo.start_transaction(&settings, "test");
+    let commit_a2 = CommitBuilder::for_rewrite_from(&settings, &commit_a)
+        .set_description("A2".to_string())
+        .write_to_repo(tx1.mut_repo());
+    tx1.mut_repo().rebase_descendants(&settings).unwrap();
+
+    let mut tx2 = repo.start_transaction(&settings, "test");
+    let commit_a3 = CommitBuilder::for_rewrite_from(&settings, &commit_a)
+        .set_description("A3".to_string())
+        .write_to_repo(tx2.mut_repo());
+    tx2.mut_repo().rebase_descendants(&settings).unwrap();
+
+    let repo = commit_transactions(&settings, vec![tx1, tx2]);
+
+    // A2 and A3 should be heads.
+    assert_eq!(
+        *repo.view().heads(),
+        hashset! {commit_a2.id().clone(), commit_a3.id().clone()}
+    );
+}
+
 #[test_case(false ; "rewrite first")]
 #[test_case(true ; "add child first")]
 fn test_merge_views_child_on_rewritten(child_first: bool) {
@@ -467,17 +491,16 @@ fn test_merge_views_child_on_rewritten(child_first: bool) {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(false);
 
-    let mut tx = test_repo.repo.start_transaction("test");
-    let commit_a =
-        testutils::create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
+    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let commit_a = create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
-    let commit_b = testutils::create_random_commit(&settings, &repo)
+    let mut tx1 = repo.start_transaction(&settings, "test");
+    let commit_b = create_random_commit(&settings, &repo)
         .set_parents(vec![commit_a.id().clone()])
         .write_to_repo(tx1.mut_repo());
 
-    let mut tx2 = repo.start_transaction("test");
+    let mut tx2 = repo.start_transaction(&settings, "test");
     let commit_a2 = CommitBuilder::for_rewrite_from(&settings, &commit_a)
         .set_description("A2".to_string())
         .write_to_repo(tx2.mut_repo());
@@ -510,21 +533,20 @@ fn test_merge_views_child_on_rewritten_divergent(on_rewritten: bool, child_first
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(false);
 
-    let mut tx = test_repo.repo.start_transaction("test");
-    let commit_a2 =
-        testutils::create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
-    let commit_a3 = testutils::create_random_commit(&settings, &test_repo.repo)
+    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let commit_a2 = create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
+    let commit_a3 = create_random_commit(&settings, &test_repo.repo)
         .set_change_id(commit_a2.change_id().clone())
         .write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
+    let mut tx1 = repo.start_transaction(&settings, "test");
     let parent = if on_rewritten { &commit_a2 } else { &commit_a3 };
-    let commit_b = testutils::create_random_commit(&settings, &repo)
+    let commit_b = create_random_commit(&settings, &repo)
         .set_parents(vec![parent.id().clone()])
         .write_to_repo(tx1.mut_repo());
 
-    let mut tx2 = repo.start_transaction("test");
+    let mut tx2 = repo.start_transaction(&settings, "test");
     let commit_a4 = CommitBuilder::for_rewrite_from(&settings, &commit_a2)
         .set_description("A4".to_string())
         .write_to_repo(tx2.mut_repo());
@@ -562,20 +584,19 @@ fn test_merge_views_child_on_abandoned(child_first: bool) {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init(false);
 
-    let mut tx = test_repo.repo.start_transaction("test");
-    let commit_a =
-        testutils::create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
-    let commit_b = testutils::create_random_commit(&settings, &test_repo.repo)
+    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let commit_a = create_random_commit(&settings, &test_repo.repo).write_to_repo(tx.mut_repo());
+    let commit_b = create_random_commit(&settings, &test_repo.repo)
         .set_parents(vec![commit_a.id().clone()])
         .write_to_repo(tx.mut_repo());
     let repo = tx.commit();
 
-    let mut tx1 = repo.start_transaction("test");
-    let commit_c = testutils::create_random_commit(&settings, &repo)
+    let mut tx1 = repo.start_transaction(&settings, "test");
+    let commit_c = create_random_commit(&settings, &repo)
         .set_parents(vec![commit_b.id().clone()])
         .write_to_repo(tx1.mut_repo());
 
-    let mut tx2 = repo.start_transaction("test");
+    let mut tx2 = repo.start_transaction(&settings, "test");
     tx2.mut_repo()
         .record_abandoned_commit(commit_b.id().clone());
     tx2.mut_repo().rebase_descendants(&settings).unwrap();
