@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -29,11 +29,10 @@ use jujutsu_lib::op_store::WorkspaceId;
 use jujutsu_lib::repo::ReadonlyRepo;
 use jujutsu_lib::repo_path::{RepoPath, RepoPathComponent, RepoPathJoin};
 use jujutsu_lib::settings::UserSettings;
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::TestWorkspace;
 use jujutsu_lib::tree_builder::TreeBuilder;
 use jujutsu_lib::working_copy::WorkingCopy;
 use test_case::test_case;
+use testutils::{create_random_commit, TestWorkspace};
 
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
@@ -70,7 +69,7 @@ fn test_checkout_file_transitions(use_git: bool) {
     let store = repo.store().clone();
     let workspace_root = test_workspace.workspace.workspace_root().clone();
 
-    #[derive(Debug, Clone, Copy)]
+    #[derive(Debug, PartialEq, Eq, Clone, Copy)]
     enum Kind {
         Missing,
         Normal,
@@ -98,21 +97,21 @@ fn test_checkout_file_transitions(use_git: bool) {
             }
             Kind::Normal => {
                 let id = testutils::write_file(store, path, "normal file contents");
-                TreeValue::Normal {
+                TreeValue::File {
                     id,
                     executable: false,
                 }
             }
             Kind::Executable => {
                 let id = testutils::write_file(store, path, "executable file contents");
-                TreeValue::Normal {
+                TreeValue::File {
                     id,
                     executable: true,
                 }
             }
             Kind::ExecutableNormalContent => {
                 let id = testutils::write_file(store, path, "normal file contents");
-                TreeValue::Normal {
+                TreeValue::File {
                     id,
                     executable: true,
                 }
@@ -123,20 +122,20 @@ fn test_checkout_file_transitions(use_git: bool) {
                 let right_file_id = testutils::write_file(store, path, "right file contents");
                 let conflict = Conflict {
                     removes: vec![ConflictPart {
-                        value: TreeValue::Normal {
+                        value: TreeValue::File {
                             id: base_file_id,
                             executable: false,
                         },
                     }],
                     adds: vec![
                         ConflictPart {
-                            value: TreeValue::Normal {
+                            value: TreeValue::File {
                                 id: left_file_id,
                                 executable: false,
                             },
                         },
                         ConflictPart {
-                            value: TreeValue::Normal {
+                            value: TreeValue::File {
                                 id: right_file_id,
                                 executable: false,
                             },
@@ -164,8 +163,8 @@ fn test_checkout_file_transitions(use_git: bool) {
                 TreeValue::Tree(id)
             }
             Kind::GitSubmodule => {
-                let mut tx = repo.start_transaction("test");
-                let id = testutils::create_random_commit(settings, repo)
+                let mut tx = repo.start_transaction(settings, "test");
+                let id = create_random_commit(settings, repo)
                     .write_to_repo(tx.mut_repo())
                     .id()
                     .clone();
@@ -396,7 +395,6 @@ fn test_checkout_discard() {
 fn test_snapshot_racy_timestamps(use_git: bool) {
     // Tests that file modifications are detected even if they happen the same
     // millisecond as the updated working copy state.
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, use_git);
     let repo = &test_workspace.repo;
@@ -428,7 +426,6 @@ fn test_snapshot_racy_timestamps(use_git: bool) {
 fn test_snapshot_special_file() {
     // Tests that we ignore when special files (such as sockets and pipes) exist on
     // disk.
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, false);
     let workspace_root = test_workspace.workspace.workspace_root().clone();
@@ -439,7 +436,7 @@ fn test_snapshot_special_file() {
     std::fs::write(&file1_disk_path, "contents".as_bytes()).unwrap();
     let file2_path = RepoPath::from_internal_string("file2");
     let file2_disk_path = file2_path.to_fs_path(&workspace_root);
-    std::fs::write(&file2_disk_path, "contents".as_bytes()).unwrap();
+    std::fs::write(file2_disk_path, "contents".as_bytes()).unwrap();
     let socket_disk_path = workspace_root.join("socket");
     UnixListener::bind(&socket_disk_path).unwrap();
     // Test the setup
@@ -485,7 +482,6 @@ fn test_snapshot_special_file() {
 fn test_gitignores(use_git: bool) {
     // Tests that .gitignore files are respected.
 
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, use_git);
     let repo = &test_workspace.repo;
@@ -561,7 +557,6 @@ fn test_gitignores_checkout_never_overwrites_ignored(use_git: bool) {
     // Tests that a .gitignore'd file doesn't get overwritten if check out a commit
     // where the file is tracked.
 
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, use_git);
     let repo = &test_workspace.repo;
@@ -599,7 +594,6 @@ fn test_gitignores_ignored_directory_already_tracked(use_git: bool) {
     // Tests that a .gitignore'd directory that already has a tracked file in it
     // does not get removed when snapshotting the working directory.
 
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, use_git);
     let repo = &test_workspace.repo;
@@ -643,7 +637,6 @@ fn test_dotgit_ignored(use_git: bool) {
     // Tests that .git directories and files are always ignored (we could accept
     // them if the backend is not git).
 
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, use_git);
     let repo = &test_workspace.repo;
@@ -676,11 +669,75 @@ fn test_dotgit_ignored(use_git: bool) {
     locked_wc.discard();
 }
 
+#[test]
+fn test_gitsubmodule() {
+    // Tests that git submodules are ignored.
+
+    let settings = testutils::user_settings();
+    let mut test_workspace = TestWorkspace::init(&settings, true);
+    let repo = &test_workspace.repo;
+    let store = repo.store().clone();
+    let workspace_root = test_workspace.workspace.workspace_root().clone();
+
+    let mut tree_builder = store.tree_builder(store.empty_tree_id().clone());
+
+    let added_path = RepoPath::from_internal_string("added");
+    let submodule_path = RepoPath::from_internal_string("submodule");
+    let added_submodule_path = RepoPath::from_internal_string("submodule/added");
+
+    tree_builder.set(
+        added_path.clone(),
+        TreeValue::File {
+            id: testutils::write_file(repo.store(), &added_path, "added\n"),
+            executable: false,
+        },
+    );
+
+    let mut tx = repo.start_transaction(&settings, "create submodule commit");
+    let submodule_id = create_random_commit(&settings, repo)
+        .write_to_repo(tx.mut_repo())
+        .id()
+        .clone();
+    tx.commit();
+
+    tree_builder.set(
+        submodule_path.clone(),
+        TreeValue::GitSubmodule(submodule_id),
+    );
+
+    let tree_id = tree_builder.write_tree();
+    let tree = store.get_tree(&RepoPath::root(), &tree_id).unwrap();
+    let wc = test_workspace.workspace.working_copy_mut();
+    wc.check_out(repo.op_id().clone(), None, &tree).unwrap();
+
+    std::fs::create_dir(submodule_path.to_fs_path(&workspace_root)).unwrap();
+
+    testutils::write_working_copy_file(
+        &workspace_root,
+        &added_submodule_path,
+        "i am a file in a submodule\n",
+    );
+
+    // Check that the files present in the submodule are not tracked
+    // when we snapshot
+    let mut locked_wc = wc.start_mutation();
+    let new_tree_id = locked_wc.snapshot(GitIgnoreFile::empty()).unwrap();
+    locked_wc.discard();
+    assert_eq!(new_tree_id, tree_id);
+
+    // Check that the files in the submodule are not deleted
+    let file_in_submodule_path = added_submodule_path.to_fs_path(&workspace_root);
+    assert!(
+        file_in_submodule_path.metadata().is_ok(),
+        "{:?} should exist",
+        file_in_submodule_path
+    );
+}
+
 #[cfg(unix)]
 #[test_case(false ; "local backend")]
 #[test_case(true ; "git backend")]
 fn test_existing_directory_symlink(use_git: bool) {
-    let _home_dir = testutils::new_user_home();
     let settings = testutils::user_settings();
     let mut test_workspace = TestWorkspace::init(&settings, use_git);
     let repo = &test_workspace.repo;
