@@ -1,4 +1,4 @@
-// Copyright 2021 Google LLC
+// Copyright 2021 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,12 +12,12 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use assert_matches::assert_matches;
 use jujutsu_lib::op_store::WorkspaceId;
 use jujutsu_lib::repo::BackendFactories;
-use jujutsu_lib::testutils;
-use jujutsu_lib::testutils::TestWorkspace;
 use jujutsu_lib::workspace::{Workspace, WorkspaceLoadError};
 use test_case::test_case;
+use testutils::TestWorkspace;
 
 #[test]
 fn test_load_bad_path() {
@@ -26,9 +26,9 @@ fn test_load_bad_path() {
     let workspace_root = temp_dir.path().to_owned();
     // We haven't created a repo in the workspace_root, so it should fail to load.
     let result = Workspace::load(&settings, &workspace_root, &BackendFactories::default());
-    assert_eq!(
+    assert_matches!(
         result.err(),
-        Some(WorkspaceLoadError::NoWorkspaceHere(workspace_root))
+        Some(WorkspaceLoadError::NoWorkspaceHere(root)) if root == workspace_root
     );
 }
 
