@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -14,12 +14,14 @@
 
 #![deny(unused_must_use)]
 
+pub mod cleanup_guard;
 pub mod cli_util;
 pub mod commands;
 pub mod config;
 pub mod diff_edit;
 pub mod formatter;
 pub mod graphlog;
+mod progress;
 pub mod template_parser;
 pub mod templater;
 pub mod ui;
