@@ -1,4 +1,4 @@
-// Copyright 2020 Google LLC
+// Copyright 2020 The Jujutsu Authors
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -12,8 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use chrono::{FixedOffset, TimeZone, Utc};
-use jujutsu_lib::backend::{CommitId, Signature};
+use chrono::{DateTime, FixedOffset, LocalResult, TimeZone, Utc};
+use jujutsu_lib::backend::{CommitId, Signature, Timestamp};
 use jujutsu_lib::commit::Commit;
 use jujutsu_lib::op_store::WorkspaceId;
 use jujutsu_lib::repo::RepoRef;
@@ -26,7 +26,7 @@ use crate::templater::{
     AuthorProperty, BranchProperty, ChangeIdProperty, CommitIdKeyword, CommitterProperty,
     ConditionalTemplate, ConflictProperty, ConstantTemplateProperty, DescriptionProperty,
     DivergentProperty, DynamicLabelTemplate, GitRefsProperty, IsGitHeadProperty,
-    IsWorkingCopyProperty, LabelTemplate, ListTemplate, LiteralTemplate, OpenProperty,
+    IsWorkingCopyProperty, LabelTemplate, ListTemplate, LiteralTemplate, SignatureTimestamp,
     StringPropertyTemplate, TagProperty, Template, TemplateFunction, TemplateProperty,
     WorkingCopiesProperty,
 };
@@ -95,17 +95,41 @@ impl TemplateProperty<Signature, String> for SignatureEmail {
     }
 }
 
-struct SignatureTimestamp;
+fn datetime_from_timestamp(context: &Timestamp) -> Option<DateTime<FixedOffset>> {
+    let utc = match Utc.timestamp_opt(
+        context.timestamp.0.div_euclid(1000),
+        (context.timestamp.0.rem_euclid(1000)) as u32 * 1000000,
+    ) {
+        LocalResult::None => {
+            return None;
+        }
+        LocalResult::Single(x) => x,
+        LocalResult::Ambiguous(y, _z) => y,
+    };
 
-impl TemplateProperty<Signature, String> for SignatureTimestamp {
-    fn extract(&self, context: &Signature) -> String {
-        let utc = Utc
-            .timestamp(
-                context.timestamp.timestamp.0.div_euclid(1000),
-                context.timestamp.timestamp.0.rem_euclid(1000) as u32 * 1000000,
-            )
-            .with_timezone(&FixedOffset::east(context.timestamp.tz_offset * 60));
-        utc.format("%Y-%m-%d %H:%M:%S.%3f %:z").to_string()
+    Some(
+        utc.with_timezone(
+            &FixedOffset::east_opt(context.tz_offset * 60)
+                .unwrap_or_else(|| FixedOffset::east_opt(0).unwrap()),
+        ),
+    )
+}
+
+struct RelativeTimestampString;
+
+impl TemplateProperty<Timestamp, String> for RelativeTimestampString {
+    fn extract(&self, context: &Timestamp) -> String {
+        datetime_from_timestamp(context)
+            .and_then(|datetime| {
+                let now = chrono::Local::now();
+
+                now.signed_duration_since(datetime).to_std().ok()
+            })
+            .map(|duration| {
+                let f = timeago::Formatter::new();
+                f.convert(duration)
+            })
+            .unwrap_or_else(|| "<out-of-range date>".to_string())
     }
 }
 
@@ -135,6 +159,10 @@ fn parse_method_chain<'a, I: 'a>(
                 let next_method = parse_signature_method(method);
                 next_method.after(property)
             }
+            Property::Timestamp(property) => {
+                let next_method = parse_timestamp_method(method);
+                next_method.after(property)
+            }
         }
     }
 }
@@ -193,18 +221,33 @@ fn parse_signature_method<'a>(method: Pair<Rule>) -> Property<'a, Signature> {
         //       `author % (name "<" email ">")`)?
         "name" => Property::String(Box::new(SignatureName)),
         "email" => Property::String(Box::new(SignatureEmail)),
-        "timestamp" => Property::String(Box::new(SignatureTimestamp)),
+        "timestamp" => Property::Timestamp(Box::new(SignatureTimestamp)),
         name => panic!("no such commit ID method: {}", name),
     };
     let chain_method = inner.last().unwrap();
     parse_method_chain(chain_method, this_function)
 }
 
+fn parse_timestamp_method<'a>(method: Pair<Rule>) -> Property<'a, Timestamp> {
+    assert_eq!(method.as_rule(), Rule::method);
+    let mut inner = method.into_inner();
+    let name = inner.next().unwrap();
+    // TODO: validate arguments
+
+    let this_function = match name.as_str() {
+        "ago" => Property::String(Box::new(RelativeTimestampString)),
+        name => panic!("no such timestamp method: {}", name),
+    };
+    let chain_method = inner.last().unwrap();
+    parse_method_chain(chain_method, this_function)
+}
+
 enum Property<'a, I> {
     String(Box<dyn TemplateProperty<I, String> + 'a>),
     Boolean(Box<dyn TemplateProperty<I, bool> + 'a>),
     CommitId(Box<dyn TemplateProperty<I, CommitId> + 'a>),
     Signature(Box<dyn TemplateProperty<I, Signature> + 'a>),
+    Timestamp(Box<dyn TemplateProperty<I, Timestamp> + 'a>),
 }
 
 impl<'a, I: 'a> Property<'a, I> {
@@ -226,6 +269,10 @@ impl<'a, I: 'a> Property<'a, I> {
                 first,
                 Box::new(move |value| property.extract(&value)),
             ))),
+            Property::Timestamp(property) => Property::Timestamp(Box::new(TemplateFunction::new(
+                first,
+                Box::new(move |value| property.extract(&value)),
+            ))),
         }
     }
 }
@@ -242,7 +289,6 @@ fn parse_commit_keyword<'a>(
         "commit_id" => Property::CommitId(Box::new(CommitIdKeyword)),
         "author" => Property::Signature(Box::new(AuthorProperty)),
         "committer" => Property::Signature(Box::new(CommitterProperty)),
-        "open" => Property::Boolean(Box::new(OpenProperty)),
         "working_copies" => Property::String(Box::new(WorkingCopiesProperty { repo })),
         "current_working_copy" => Property::Boolean(Box::new(IsWorkingCopyProperty {
             repo,
@@ -276,6 +322,13 @@ fn coerce_to_string<'a, I: 'a>(
             property,
             Box::new(|signature| signature.name),
         )),
+        Property::Timestamp(property) => Box::new(TemplateFunction::new(
+            property,
+            Box::new(|timestamp| match datetime_from_timestamp(&timestamp) {
+                Some(datetime) => datetime.format("%Y-%m-%d %H:%M:%S.%3f %:z").to_string(),
+                None => "<out-of-range date>".to_string(),
+            }),
+        )),
     }
 }
 
@@ -291,6 +344,10 @@ fn parse_boolean_commit_property<'a>(
     match pair.as_rule() {
         Rule::identifier => match parse_commit_keyword(repo, workspace_id, pair.clone()).0 {
             Property::Boolean(property) => property,
+            Property::String(property) => Box::new(TemplateFunction::new(
+                property,
+                Box::new(|string| !string.is_empty()),
+            )),
             _ => panic!("cannot yet use this as boolean: {:?}", pair),
         },
         _ => panic!("cannot yet use this as boolean: {:?}", pair),
@@ -357,12 +414,9 @@ fn parse_commit_term<'a>(
                         let content: Box<dyn Template<Commit> + 'a> =
                             parse_commit_template_rule(repo, workspace_id, arg_template);
                         let get_labels = move |commit: &Commit| -> String {
-                            let mut buf: Vec<u8> = vec![];
-                            {
-                                let writer = Box::new(&mut buf);
-                                let mut formatter = PlainTextFormatter::new(writer);
-                                label_template.format(commit, &mut formatter).unwrap();
-                            }
+                            let mut buf = vec![];
+                            let mut formatter = PlainTextFormatter::new(&mut buf);
+                            label_template.format(commit, &mut formatter).unwrap();
                             String::from_utf8(buf).unwrap()
                         };
                         Box::new(DynamicLabelTemplate::new(content, Box::new(get_labels)))
@@ -431,8 +485,9 @@ pub fn parse_commit_template<'a>(
     let first_pair = pairs.next().unwrap();
     assert!(pairs.next().is_none());
 
-    assert!(
-        first_pair.as_span().end() == template_text.len(),
+    assert_eq!(
+        first_pair.as_span().end(),
+        template_text.len(),
         "failed to parse template past position {}",
         first_pair.as_span().end()
     );
